# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2018, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#==========================================================================
import logging, os, time
from datetime import datetime
from .naeusb import packuint32
from chipwhisperer.capture.utils.programming_files import FileReader

XMEGAMEM_TYPE_APP = 1
XMEGAMEM_TYPE_BOOT = 2
XMEGAMEM_TYPE_EEPROM = 3
XMEGAMEM_TYPE_FUSE = 4
XMEGAMEM_TYPE_LOCKBITS = 5
XMEGAMEM_TYPE_USERSIG = 6
XMEGAMEM_TYPE_FACTORY_CALIBRATION = 7

# NOTE: These objects are currently manually maintained. Eventually it will be automatically created
#      from avrdude.conf, but I'd like to test with a few more devices before doing that.

class XMEGADummy(object):
    signature = [0x00, 0x00, 0x00]
    name = "Unknown XMEGA device"

    memtypes = {
        "signature":{"offset":0x1000090, "size":3},
    }

class XMEGA16A4(object):
    signature = [0x1e, 0x94, 0x41]
    name = "XMEGA16A4"

    memtypes = {
       "signature":{"offset":0x1000090, "size":3},
       "flash":{"offset":0x0800000, "size":0x00005000, "pagesize":0x100, "type":XMEGAMEM_TYPE_APP},
       "eeprom":{"offset":0x08c0000, "size":0x0400, "pagesize":0x20, "readsize":0x100, "type":XMEGAMEM_TYPE_EEPROM},
       "fuse1":{"offset":0x8f0021, "size":1},
       "fuse2":{"offset":0x8f0022, "size":1},
       "fuse4":{"offset":0x8f0024, "size":1},
       "fuse5":{"offset":0x8f0025, "size":1},
     }

class XMEGA128A4U(object):
    signature = [0x1e, 0x97, 0x46]
    name = "XMEGA128A4U"

    memtypes = {
       "signature":{"offset":0x1000090, "size":3},
       "flash":{"offset":0x0800000, "size":0x00022000, "pagesize":0x100, "type":XMEGAMEM_TYPE_APP},
       "eeprom":{"offset":0x08c0000, "size":0x0800, "pagesize":0x20, "readsize":0x100, "type":XMEGAMEM_TYPE_EEPROM},
       "fuse1":{"offset":0x8f0021, "size":1},
       "fuse2":{"offset":0x8f0022, "size":1},
       "fuse4":{"offset":0x8f0024, "size":1},
       "fuse5":{"offset":0x8f0025, "size":1},
     }

class XMEGA128D4(object):
    signature = [0x1e, 0x97, 0x47]
    name = "XMEGA128D4"

    memtypes = {
       "signature":{"offset":0x1000090, "size":3},
       "flash":{"offset":0x0800000, "size":0x00022000, "pagesize":0x100, "type":XMEGAMEM_TYPE_APP},
       "eeprom":{"offset":0x08c0000, "size":0x0800, "pagesize":0x20, "readsize":0x100, "type":XMEGAMEM_TYPE_EEPROM},
       "fuse1":{"offset":0x8f0021, "size":1},
       "fuse2":{"offset":0x8f0022, "size":1},
       "fuse4":{"offset":0x8f0024, "size":1},
       "fuse5":{"offset":0x8f0025, "size":1},
     }

class XMEGA128A3U(object):
    signature = [0x1e, 0x97, 0x42]
    name = "XMEGA128A3U"

    memtypes = {
       "signature":{"offset":0x1000090, "size":3},
       "flash":{"offset":0x0800000, "size":0x00022000, "pagesize":0x100, "type":XMEGAMEM_TYPE_APP},
       "eeprom":{"offset":0x08c0000, "size":0x0800, "pagesize":0x20, "readsize":0x100, "type":XMEGAMEM_TYPE_EEPROM},
       "fuse1":{"offset":0x8f0021, "size":1},
       "fuse2":{"offset":0x8f0022, "size":1},
       "fuse4":{"offset":0x8f0024, "size":1},
       "fuse5":{"offset":0x8f0025, "size":1},
     }

supported_xmega = [XMEGA128A4U(), XMEGA128D4(), XMEGA16A4(), XMEGA128A3U()]

def print_fun(s):
    print(s)

class XMEGAPDI(object):
    """
    Class for programming an XMEGA device using the NAEUSB Firmware in the ChipWhisperer 1173/1200.
    """

    CMD_XMEGA_PROGRAM = 0x20
    """USB Command for XMEGA Programming (PDI Interface)"""

    XPROG_CMD_ENTER_PROGMODE = 0x01
    """PDI Command: Enable PDI, enter programming mode"""

    XPROG_CMD_LEAVE_PROGMODE = 0x02
    """PDI Command: Leave programming mode, disable PDI"""

    XPROG_CMD_ERASE = 0x03
    """PDI Command: Erase command, specify what to erase as well"""

    XPROG_CMD_WRITE_MEM = 0x04
    """PDI Command: Write data from RAMBUF to XMEGA memory"""

    XPROG_CMD_READ_MEM = 0x05
    """PDI Command: Read data from XMEGA memory to RAMBUF"""

    XPROG_CMD_CRC = 0x06
    """PDI Command: Perform CRC of Memory"""

    XPROG_CMD_SET_PARAM = 0x07
    """PDI Command: Set programming parameters"""

    XPROG_GET_STATUS = 0x20
    """PDI Command: Get status of last command"""

    XPROG_GET_RAMBUF = 0x21
    """PDI Command: Read data from RAMBUF"""

    XPROG_SET_RAMBUF = 0x22
    """PDI Command: Write data to RAMBUF"""

    XPROG_PARAM_TIMEOUT = 0x08
    """Parameter: Timeout"""


    # Chip Erase Types
    XPROG_ERASE_CHIP = 1
    """Erase type: Chip"""

    XPROG_ERASE_APP = 2
    """Erase type: Application Section"""

    # Maximum size of buffer in our system
    MAX_BUFFER_SIZE = 256

    def __init__(self, usb, timeout=200):
        """
        Set the USB communications instance.
        """

        self._usb = usb
        self._timeout = timeout
        self._pdienabled = False
        self._chip = XMEGADummy()
        self.lastFlashedFile = "unknown"

#### HIGH LEVEL FUNCTIONS
    def find(self):
        self.setParamTimeout(100)
        self.enablePDI(True)

        # Read signature bytes
        data = self.readMemory(0x01000090, 3, "signature")

        # Check if it's one we know about?
        for t in supported_xmega:
            if ((data[0] == t.signature[0]) and
                (data[1] == t.signature[1]) and
                (data[2] == t.signature[2])):

                logging.debug("Detected known XMEGA: %s"%t.name)
                self.setChip(t)
                return data, t

        #No known chip found?
        logging.debug("Detected unknown XMEGA: %02x %02x %02x"%(data[0], data[1], data[2]))
        return data, None

    def erase(self, memtype="chip"):
        if memtype == "app":
            self.eraseApp()
        elif memtype == "chip":
            self.eraseChip()
        else:
            raise ValueError("Invalid memtype: %s" % memtype)


    def autoProgram(self, hexfile, erase=True, verify=True, logfunc=print_fun, waitfunc=None):
        """Helper funciton for GUI, auto-programs XMEGA device while printing messages to different options. Returns true/false."""

        status = "FAILED"

        fname = hexfile
        if logfunc: logfunc("***Starting FLASH program process at %s***" % datetime.now().strftime('%H:%M:%S'))
        if waitfunc: waitfunc()
        if os.path.isfile(fname):
            if logfunc: logfunc("File %s last changed on %s" % (fname, time.ctime(os.path.getmtime(fname))))

            try:
                if logfunc: logfunc("Entering Programming Mode")
                if waitfunc: waitfunc()
                self.find()

                if erase:
                    try:
                        self.erase()
                    except IOError:
                        if logfunc: logfunc("**chip-erase timeout, erasing application only**")
                        if waitfunc: waitfunc()
                        self.enablePDI(False)
                        self.enablePDI(True)
                        self.erase("app")

                if waitfunc: waitfunc()
                self.program(hexfile, memtype="flash", verify=verify, logfunc=logfunc, waitfunc=waitfunc)
                if waitfunc: waitfunc()
                if logfunc: logfunc("Exiting programming mode")
                self.close()
                if waitfunc: waitfunc()

                status = "SUCCEEDED"

            except IOError as e:
                if logfunc: logfunc("FAILED: %s" % str(e))
                try:
                    self.close()
                except IOError:
                    pass

        else:
            if logfunc: logfunc("%s does not appear to be a file, check path" % fname)

        if logfunc: logfunc("***FLASH Program %s at %s***" % (status, datetime.now().strftime('%H:%M:%S')))

        return status == "SUCCEEDED"

    def program(self, filename, memtype="flash", verify=True, logfunc=print_fun, waitfunc=None):
        """Programs memory type, dealing with opening filename as either .hex or .bin file"""
        self.lastFlashedFile = filename

        fdata, fsize = FileReader(filename)

        startaddr = self._chip.memtypes[memtype]["offset"]
        maxsize = self._chip.memtypes[memtype]["size"]

        if fsize > maxsize:
            raise IOError("File %s appears to be %d bytes, larger than %s size of %d" % (filename, fsize, memtype, maxsize))

        logfunc("XMEGA Programming %s..." % memtype)
        if waitfunc: waitfunc()
        self.writeMemory(startaddr, fdata, memtype)  # , erasePage=True

        logfunc("XMEGA Reading %s..." % memtype)
        if waitfunc: waitfunc()
        # Do verify run
        rdata = self.readMemory(startaddr, len(fdata), memtype)

        for i in range(0, len(fdata)):
            if fdata[i] != rdata[i]:
                raise IOError("Verify failed at 0x%04x, %x != %x" % (i, fdata[i], rdata[i]))

        logfunc("Verified %s OK, %d bytes" % (memtype, fsize))

#### LOW LEVEL FUNCTIONS

    def close(self):
        self.enablePDI(False)


    def setParamTimeout(self, timeoutMS):
        """
        Set timeout for executing PDI commands, sets timeout both on NAEUSB chip and host-side USB API.
        """

        self._timeout = timeoutMS + 50
        timeoutticks = int((float(timeoutMS) / 1000.0) * 2500.0)
        pload = [self.XPROG_PARAM_TIMEOUT]
        pload.extend(packuint32(timeoutticks))
        self._xmegaDoWrite(self.XPROG_CMD_SET_PARAM, pload)


    def enablePDI(self, status):
        """
        Enable or disable PDI interface and prepare XMEGA chip for new status, either entering or exiting
        programming mode.

        Raises IOError() if an error occurs (such as no chip found).
        """

        self._pdienabled = False

        if status:
            # self._xmegaDoWrite(self.XPROG_CMD_LEAVE_PROGMODE)
            self._xmegaDoWrite(self.XPROG_CMD_ENTER_PROGMODE)
            self._pdienabled = True
        else:
            self._xmegaDoWrite(self.XPROG_CMD_LEAVE_PROGMODE)

    def validate_mode(self):
        if not self._pdienabled:
            raise IOError("Enable PDI mode first")

    def readMemory(self, addr, dlen, memname="flash"):
        """
        Read memory such as FLASH or EEPROM. Can specify an arbitrary length of data.

        Args:
            addr (int): Address to read from.
            dlen (in): How much data to read.
        Kwargs:
            memname (str): Type of memory, such as "flash" or "eeprom". Defaults to 'flash', but
                           this will often work for other memory types.
        Returns:
            list.
        Raises:
            IOError
        """

        self.validate_mode()

        memspec = self._chip.memtypes[memname]

        memread = 0
        endptsize = 64
        # start = 0
        # end = endptsize

        if "readsize" in list(memspec.keys()):
            readsize = memspec["readsize"]
        elif "pagesize" in list(memspec.keys()):
            readsize = memspec["pagesize"]
        else:
            readsize = dlen

        membuf = []

        while memread < dlen:

            #Read into internal buffer
            ramreadln = dlen - memread

            # Check if maximum size for memory type
            if ramreadln > readsize:
                ramreadln = readsize

            # Check if maximum size for internal buffer
            if ramreadln > self.MAX_BUFFER_SIZE:
                ramreadln = self.MAX_BUFFER_SIZE

            infoblock = [0]  # memspec["type"]
            infoblock.extend(packuint32(addr + memread))
            infoblock.append(ramreadln & 0xff)
            infoblock.append((ramreadln >> 8) & 0xff)

            self._xmegaDoWrite(self.XPROG_CMD_READ_MEM, data=infoblock)

            epread = 0

            # First we need to fill the page buffer in the USB Interface using smaller transactions
            while epread < ramreadln:

                epreadln = ramreadln - epread
                if epreadln > endptsize:
                    epreadln = endptsize

                # Read data out progressively
                membuf.extend(self._xmegaDoRead(self.XPROG_GET_RAMBUF | (epread << 8), dlen=epreadln))

                # print epread

                epread += epreadln

            memread += ramreadln

        return membuf

    def writeMemory(self, addr, data, memname, erasePage=False, programPage=True):
        """
        Write memory such as FLASH or EEPROM. Can specify an arbitrary length of data.

        Args:
            addr (int): Address to write at, should be page aligned if writing paged memory!
            data (list): Data to write.
            memname (str): Type of memory, such as "flash" or "eeprom".
        Kwargs:
            erasePage (bool): Should we perform a page erase before writing? Defaults to FALSE.
            programPage (bool): Should we perform a page write once memory copied? Defaults to TRUE.
                                If writing internal RAM set this to FALSE, but for writing FLASH/
                                EEPROM leave as TRUE.
        Raises:
            IOError
        """

        self.validate_mode()

        PAGEMODE_WRITE = (1 << 1)
        PAGEMODE_ERASE = (1 << 0)

        memspec = self._chip.memtypes[memname]

        memwritten = 0
        endptsize = 64
        start = 0
        end = endptsize
        pagesize = memspec["pagesize"]

        if addr % pagesize:
            logging.warning('You appear to be writing to an address that is not page aligned, you will probably write the wrong data')

        while memwritten < len(data):

            epwritten = 0

            # First we need to fill the page buffer in the USB Interface using smaller transactions
            while epwritten < pagesize:

                # Check for less than full endpoint written
                if end > len(data):
                    end = len(data)

                # Get slice of data
                epdata = data[start:end]

                # print "%d %d %d" % (epwritten, len(epdata), memwritten)
                # Copy to USB interface buffer
                self._xmegaDoWrite(self.XPROG_SET_RAMBUF | (epwritten << 8), data=epdata, checkStatus=False)

                epwritten += len(epdata)

                # Check for final write indicating we are done
                if end == len(data):
                    break

                start += endptsize
                end += endptsize

            # Copy internal buffer to final location (probably FLASH memory)

            if not ("type" in list(memspec.keys())):
                raise IOError("Write on memory type that doesn't have 'type', probably read-only?")

            # Do write into memory type
            infoblock = [memspec["type"], 0]

            if programPage:
                infoblock[1] |= PAGEMODE_WRITE

            if erasePage:
                infoblock[1] |= PAGEMODE_ERASE

            infoblock.extend(packuint32(addr + memwritten))
            infoblock.append(epwritten & 0xff)
            infoblock.append((epwritten >> 8) & 0xff)

            # print "%x" % (addr + memwritten)
            # print epwritten

            self._xmegaDoWrite(self.XPROG_CMD_WRITE_MEM, data=infoblock)

            memwritten += epwritten

    def eraseChip(self):
        self.validate_mode()
        self._xmegaDoWrite(self.XPROG_CMD_ERASE, data=[self.XPROG_ERASE_CHIP, 0, 0, 0, 0])

    def eraseApp(self):
        self.validate_mode()
        self._xmegaDoWrite(self.XPROG_CMD_ERASE, data=[self.XPROG_ERASE_APP, 0, 0, 0, 0])

    def setChip(self, chiptype):
        self._chip = chiptype


#### INTERNAL FUNCTIONS

    def _xmegaDoWrite(self, cmd, data=[], checkStatus=True):
        """
        Send a command to the PDI programming interface, optionally check if command executed OK, and if not
        raise IOError()
        """

        # windex selects interface
        self._usb.sendCtrl(self.CMD_XMEGA_PROGRAM, cmd, data)

        # Check status
        if checkStatus:
            status = self._xmegaDoRead(cmd=0x0020, dlen=3)
            if status[1] != 0x00:
                raise IOError("XMEGA Command %x failed: err=%x, timeout=%d" % (status[0], status[1], status[2]))

    def _xmegaDoRead(self, cmd, dlen=1):
        """
        Read the result of some command.
        """
        # windex selects interface, set to 0
        return self._usb.readCtrl(self.CMD_XMEGA_PROGRAM, cmd, dlen)


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import numpy as np
from chipwhisperer.common.utils.util import camel_case_deprecated
from collections import OrderedDict


class Results(object):
    """
    Results type used for attacks generating peaks indicating the 'best' success. Examples include
    standard DPA & CPA attacks.
    """

    def __init__(self, numSubkeys=16, numPerms=256):
        self.numSubkeys = numSubkeys
        self.numPerms = numPerms
        self.known_key = None
        self.clear()

    def best_guesses(self):
        """ Gets best subkey guesses from attack results

        Returns:
            List of OrderedDicts with keys 'guess', 'correlation' and 'pge'.
        """
        guess_list = []
        stats = self.find_maximums()
        for i, subkey in enumerate(stats):
            dict = OrderedDict()
            dict['guess'] = subkey[0][0]
            dict['correlation'] = subkey[0][2]
            dict['pge'] = self.simple_PGE(i)
            guess_list.append(dict)

        return guess_list

    def __str__(self):
        ret = ""
        ret += "Subkey KGuess Correlation\n"
        guesses = self.best_guesses()
        for i,subkey in enumerate(guesses):
            ret += "  {:02d}    0x{:02X}    {:7.5f}\n".format(i, subkey['guess'], subkey['correlation'])
        return ret

    def clear(self):
        #Diffs from CPA/DPA Attack
        self.diffs = [None]*self.numSubkeys

        #Maximum diff & location of maximum
        self.maxes = [0]*self.numSubkeys
        for i in range(0, self.numSubkeys):
            self.maxes[i] = np.zeros(self.numPerms, dtype=[('hyp', 'i2'), ('point', 'i4'), ('value', 'f8')])

        #If maximum diffs are valid & sorted correctly
        self.maxValid = [False]*self.numSubkeys
        self.pge = [255]*self.numSubkeys
        self.diffs_tnum = [None]*self.numSubkeys
        self.pge_total = []
        self.maxes_list = [list() for i in range(0, self.numSubkeys)]

        #TODO: Ensure this gets called by attack algorithms when rerunning

    def simple_PGE(self, bnum):
        """Returns the partial guessing entropy of subkey."""
        if self.maxValid[bnum] == False:
            #TODO: should sort
            return 1
        return self.pge[bnum]

    simplePGE = camel_case_deprecated(simple_PGE)

    def set_known_key(self, known_key):
        """Sets the known encryption key."""
        self.known_key = known_key

    setKnownkey = camel_case_deprecated(set_known_key)

    def update_subkey(self, bnum, data, copy=True, force_update=False, tnum=None):
        """Update the specific subkey.

        Args:
            bnum (int): The index of the subkey.
            data (int): The new subkey byte.
            copy (int):
        """
        if (id(data) != id(self.diffs[bnum])) or force_update:
            self.maxValid[bnum] = False

            if data is not None and copy:
                self.diffs[bnum] = data[:]
                self.diffs_tnum[bnum] = tnum
            else:
                self.diffs[bnum] = data
                self.diffs_tnum[bnum] = tnum

    updateSubkey = camel_case_deprecated(update_subkey)

    def find_key(self, use_absolute=True):
        """ Find the best guess for the key from the attack.

        Args:
            use_absolute (bool, optional): Use the absolute value of the
                correlation during analysis.

        Returns:
            The best guess for a key from the attack as a list.
        """
        res = self.find_maximums(use_absolute=use_absolute)
        return [subkey[0][0] for subkey in res]

    def find_maximums(self, bytelist=None, use_absolute=True, use_single=False):
        """Information from the attack:

        Args:
            bytelist (list): Iterable of subkeys to compute and organize results
                for.
            use_absolute (bool): Use absolute value of correlation to find highest
                correlation.
            use_single (bool): All table values are taken from the same point the
                maximum is taken from.


        Returns:
            list: Ordered by subkey index::

                [subkey0_data, subkey1_data, subkey2_data, ...]

            *subkey0_data* is another list containing guesses ordered by strength
            of correlation::

                [guess0, guess1, guess2, ...]

            *guess0* is a tuple containing::

                (key_guess, location_of_max, correlation)

        For example, if you want to print the correlation of the best guess
        of the 4th subkey, you would run::

            print(attack_results.find_maximums()[4][0][2])

        Note the "point location of the max" is normally not calculated/tracked,
        and thus returns as a 0.
        """
        if bytelist is None:
            bytelist = list(range(0, self.numSubkeys))

        # print useAbsolute

        for i in bytelist:
            if self.diffs[i] is None:
                self.maxValid[i] = False
                continue

            if self.maxValid[i] == False:
                for hyp in range(0, self.numPerms):
                    if use_absolute:
                        v = np.nanmax(np.fabs(self.diffs[i][hyp]))
                    else:
                        v = np.nanmax(self.diffs[i][hyp])

                    mvalue = v

                    #Get maximum value for this hypothesis
                    try:
                        mindex = np.amin(np.where(v == mvalue))
                    except ValueError:
                        mindex = self.numPerms-1
                    self.maxes[i][hyp]['hyp'] = hyp
                    self.maxes[i][hyp]['point'] = mindex
                    self.maxes[i][hyp]['value'] = mvalue

                #TODO: why does this fail?
                #self.maxes[i][np.isnan(self.maxes[i]['value'])]['value'] = 0
                #TODO: workaround for PGE, as NaN's get ranked first
                numnans = np.isnan(self.maxes[i]['value']).sum()

                if use_single:
                    #All table values are taken from same point MAX is taken from
                    where = self.maxes[i][0]['point']
                    for j in range(0, self.numPerms):
                        self.maxes[i][j]['point'] = where
                        self.maxes[i][j]['value'] = self.diffs[i][self.maxes[i][j]['hyp']][where]

                self.maxes[i][::-1].sort(order='value') # sorts nunpy array in place and in reverse order
                self.maxValid[i] = True

                if self.known_key is not None:
                    try:
                        self.pge[i] = np.where(self.maxes[i]['hyp'] == self.known_key[i])[0][0] - numnans
                        if self.pge[i] < 0:
                            self.pge[i] = self.numPerms/2
                    except IndexError:
                        self.pge[i] = self.numPerms-1

            tnum = self.diffs_tnum[i]
            self.pge_total.append({'trace':tnum, 'subkey':i, 'pge':self.pge[i]})

            if len(self.maxes_list[i]) == 0 or self.maxes_list[i][-1]['trace'] != tnum:
                self.maxes_list[i].append({'trace':tnum, 'maxes':np.array(self.maxes[i])})

        return self.maxes

    findMaximums = camel_case_deprecated(find_maximums)



#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
from collections import OrderedDict
import inspect
from .base import ModelsBase
import numpy as np
from chipwhisperer.common.utils.util import binarylist2bytearray, bytearray2binarylist

class DESLeakageHelper(object):

    #Name of DES Model
    name = 'DES Leakage Model (unnamed)'

    #c model enumeration value, if a C model exists for this device
    c_model_enum_value = None
    c_model_enum_name = None

    # Based on https://gist.github.com/eigenein/1275094
    # Author:   Todd Whiteman
    # Date:     16th March, 2009
    # Verion:   2.0.0
    # License:  Public Domain - free to do as you wish
    # Homepage: http://twhiteman.netfirms.com/des.html

    sBox = 8*[64*[0]]

    sBox[0] = (14,  4, 13,  1,  2, 15, 11,  8,  3, 10,  6, 12,  5,  9,  0,  7,
                0, 15,  7,  4, 14,  2, 13,  1, 10,  6, 12, 11,  9,  5,  3,  8,
                4,  1, 14,  8, 13,  6,  2, 11, 15, 12,  9,  7,  3, 10,  5,  0,
               15, 12,  8,  2,  4,  9,  1,  7,  5, 11,  3, 14, 10,  0,  6, 13)

    sBox[1] = (15,  1,  8, 14,  6, 11,  3,  4,  9,  7,  2, 13, 12,  0,  5, 10,
                3, 13,  4,  7, 15,  2,  8, 14, 12,  0,  1, 10,  6,  9, 11,  5,
                0, 14,  7, 11, 10,  4, 13,  1,  5,  8, 12,  6,  9,  3,  2, 15,
               13,  8, 10,  1,  3, 15,  4,  2, 11,  6,  7, 12,  0,  5, 14,  9)

    sBox[2] = (10,  0,  9, 14,  6,  3, 15,  5,  1, 13, 12,  7, 11,  4,  2,  8,
               13,  7,  0,  9,  3,  4,  6, 10,  2,  8,  5, 14, 12, 11, 15,  1,
               13,  6,  4,  9,  8, 15,  3,  0, 11,  1,  2, 12,  5, 10, 14,  7,
                1, 10, 13,  0,  6,  9,  8,  7,  4, 15, 14,  3, 11,  5,  2, 12)

    sBox[3] = ( 7, 13, 14,  3,  0,  6,  9, 10,  1,  2,  8,  5, 11, 12,  4, 15,
               13,  8, 11,  5,  6, 15,  0,  3,  4,  7,  2, 12,  1, 10, 14,  9,
               10,  6,  9,  0, 12, 11,  7, 13, 15,  1,  3, 14,  5,  2,  8,  4,
                3, 15,  0,  6, 10,  1, 13,  8,  9,  4,  5, 11, 12,  7,  2, 14)

    sBox[4] = ( 2, 12,  4,  1,  7, 10, 11,  6,  8,  5,  3, 15, 13,  0, 14,  9,
               14, 11,  2, 12,  4,  7, 13,  1,  5,  0, 15, 10,  3,  9,  8,  6,
                4,  2,  1, 11, 10, 13,  7,  8, 15,  9, 12,  5,  6,  3,  0, 14,
               11,  8, 12,  7,  1, 14,  2, 13,  6, 15,  0,  9, 10,  4,  5,  3)

    sBox[5] = (12,  1, 10, 15,  9,  2,  6,  8,  0, 13,  3,  4, 14,  7,  5, 11,
               10, 15,  4,  2,  7, 12,  9,  5,  6,  1, 13, 14,  0, 11,  3,  8,
                9, 14, 15,  5,  2,  8, 12,  3,  7,  0,  4, 10,  1, 13, 11,  6,
                4,  3,  2, 12,  9,  5, 15, 10, 11, 14,  1,  7,  6,  0,  8, 13)

    sBox[6] = ( 4, 11,  2, 14, 15,  0,  8, 13,  3, 12,  9,  7,  5, 10,  6,  1,
               13,  0, 11,  7,  4,  9,  1, 10, 14,  3,  5, 12,  2, 15,  8,  6,
                1,  4, 11, 13, 12,  3,  7, 14, 10, 15,  6,  8,  0,  5,  9,  2,
                6, 11, 13,  8,  1,  4, 10,  7,  9,  5,  0, 15, 14,  2,  3, 12)

    sBox[7] = (13,  2,  8,  4,  6, 15, 11,  1, 10,  9,  3, 14,  5,  0, 12,  7,
                1, 15, 13,  8, 10,  3,  7,  4, 12,  5,  6, 11,  0, 14,  9,  2,
                7, 11,  4,  1,  9, 12, 14,  2,  0,  6, 10, 13, 15,  3,  5,  8,
                2,  1, 14,  7,  4, 10,  8, 13, 15, 12,  9,  0,  3,  5,  6, 11)


    # Permutation and translation tables for DES
    __pc1 = [56, 48, 40, 32, 24, 16,  8,
          0, 57, 49, 41, 33, 25, 17,
          9,  1, 58, 50, 42, 34, 26,
         18, 10,  2, 59, 51, 43, 35,
         62, 54, 46, 38, 30, 22, 14,
          6, 61, 53, 45, 37, 29, 21,
         13,  5, 60, 52, 44, 36, 28,
         20, 12,  4, 27, 19, 11,  3
    ]

    __pc1_inv = [
        7, 15, 23, 55, 51, 43, 35, None,
        6, 14, 22, 54, 50, 42, 34, None,
        5, 13, 21, 53, 49, 41, 33, None,
        4, 12, 20, 52, 48, 40, 32, None,
        3, 11, 19, 27, 47, 39, 31, None,
        2, 10, 18, 26, 46, 38, 30, None,
        1, 9, 17, 25, 45, 37, 29, None,
        0, 8, 16, 24, 44, 36, 28, None
    ]

    # permuted choice key (table 2)
    __pc2 = [
        13, 16, 10, 23,  0,  4,
         2, 27, 14,  5, 20,  9,
        22, 18, 11,  3, 25,  7,
        15,  6, 26, 19, 12,  1,
        40, 51, 30, 36, 46, 54,
        29, 39, 50, 44, 32, 47,
        43, 48, 38, 55, 33, 52,
        45, 41, 49, 35, 28, 31
    ]

    __pc2_inv = [
        4, 23, 6, 15, 5, 9, 19,
        17, None, 11, 2, 14, 22,
        0, 8, 18, 1, None, 13, 21,
        10, None, 12, 3, None, 16,
        20, 7, 46, 30, 26, 47,
        34, 40, None, 45, 27, None,
        38, 31, 24, 43, None, 36,
        33, 42, 28, 35, 37, 44,
        32, 25, 41, None, 29, 39
    ]

    # initial permutation IP
    __ip = [57, 49, 41, 33, 25, 17, 9,  1,
        59, 51, 43, 35, 27, 19, 11, 3,
        61, 53, 45, 37, 29, 21, 13, 5,
        63, 55, 47, 39, 31, 23, 15, 7,
        56, 48, 40, 32, 24, 16, 8,  0,
        58, 50, 42, 34, 26, 18, 10, 2,
        60, 52, 44, 36, 28, 20, 12, 4,
        62, 54, 46, 38, 30, 22, 14, 6
    ]

    # Expansion table for turning 32 bit blocks into 48 bits
    __expansion_table = [
        31,  0,  1,  2,  3,  4,
         3,  4,  5,  6,  7,  8,
         7,  8,  9, 10, 11, 12,
        11, 12, 13, 14, 15, 16,
        15, 16, 17, 18, 19, 20,
        19, 20, 21, 22, 23, 24,
        23, 24, 25, 26, 27, 28,
        27, 28, 29, 30, 31,  0
    ]

    # number left rotations of pc1
    __left_rotations = [0, 1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1]

    def __permutate(self, table, block):
        """Permutate this block with the specified table"""
        return [block[v] if v is not None else v for i,v in enumerate(table)]

    def keyScheduleRounds(self, inputkey, inputround, desiredround, returnSubkeys=True):
        """Create the 16 subkeys K[1] to K[16] from the given key"""
        if inputround == 0:
            inputkey = bytearray2binarylist(inputkey,8)
            key = self.__permutate(self.__pc1, inputkey)
        else:
            inputkey = bytearray2binarylist(inputkey,6)
            key = self.__permutate(self.__pc2_inv, inputkey)
        i = inputround
        L = key[:28]
        R = key[28:]
        while i < desiredround:
            i += 1
            j = 0
            # Perform circular left shifts
            while j < self.__left_rotations[i]:
                L.append(L[0])
                del L[0]

                R.append(R[0])
                del R[0]

                j += 1
        while i > desiredround:
            # Perform circular right shifts
            j = 0
            while j < self.__left_rotations[i]:
                L.insert(0,L[27])
                del L[28]

                R.insert(0,R[27])
                del R[28]

                j += 1
            i -= 1
        if desiredround==0:
            key = self.__permutate(self.__pc1_inv, L + R)
            return binarylist2bytearray(key, 8) if returnSubkeys else key
        else:
            key = self.__permutate(self.__pc2, L + R)
            return binarylist2bytearray(key, 6) if returnSubkeys else key

    def sbox_in_first_fbox(self, pt, guess, bnum):
        init=bytearray2binarylist(pt, 8)

        initPermut = self.__permutate(self.__ip, init)
        R = initPermut[32:]
        expR = self.__permutate(self.__expansion_table, R)
        expR = expR[bnum*6:(1+bnum)*6]
        guess = np.unpackbits(np.uint8(guess))
        guess = guess[2:]
        B = list(map(lambda x, y: x ^ y, expR, guess))
        return B

    def sbox_out_first_fbox(self, pt, guess, bnum):
        B = self.sbox_in_first_fbox(pt, guess, bnum)
        m = (B[0] << 1) + B[5]
        n = (B[1] << 3) + (B[2] << 2) + (B[3] << 1) + B[4]
        # Find the permutation value
        return self.sBox[bnum][(m << 4) + n]

    def leakage(self, pt, ct, key, bnum):
        """
        Override this function with specific leakage function (S-Box output, HD, etc).

        Args:
            pt: 16-byte plain-text input
            ct: 16-byte cipher-text output.
            key: 16-byte AES key - byte 'bnum' may be a GUESS if key is known. Rest of bytes may/may not be valid too.
            bnum: Byte number we are trying to attack.

        Returns:
            Value that will be presented on the 8-bit bus. Leakage model (such as HW) will map this to leakage itself.
        """
        raise NotImplementedError("ASKLeakageHelper does not implement leakage")

class SBox_output(DESLeakageHelper):
    name = 'HW: SBoxes Output, First Round'
    c_model_enum_value = 0
    c_model_enum_name = 'LEAK_HW_SBOXOUT_FIRSTROUND'
    def leakage(self, pt, ct, key, bnum):
        return self.sbox_out_first_fbox(pt, key[bnum], bnum)

class SBox_input(DESLeakageHelper):
    name = 'HW: SBoxes Input, First Round'
    c_model_enum_value = 1
    c_model_enum_name = 'LEAK_HW_SBOXIN_FIRSTROUND'
    def leakage(self, pt, ct, key, bnum):
        return binarylist2bytearray(self.sbox_in_first_fbox(pt, key[bnum], bnum), 6)[0]

enc_list = [SBox_output, SBox_input]
dec_list = []

class DES(ModelsBase):
    _name = 'DES'

    hwModels = OrderedDict((mod.name, mod) for mod in (enc_list + dec_list))

    def __init__(self, model=SBox_output, bitmask=0xFF):
        ModelsBase.__init__(self, 8, 64, model=model)
        self.numRoundKeys = 16
        self._mask = bitmask

    def _updateHwModel(self):
        """" Re-implement this to update leakage model """
        self.modelobj = None

        #Check if they passed an object...
        if isinstance(self.model, DESLeakageHelper):
            self.modelobj = self.model

        #Check if they passed a class...
        elif inspect.isclass(self.model) and issubclass(self.model, DESLeakageHelper):
            self.modelobj = self.model()

        #Otherwise it's probably one of these older keys (kept for backwards-compatability)
        else:
            for mod in self.hwModels:
                if (mod.c_model_enum_value == self.model) or (mod.name == self.model):
                    self.modelobj = mod()
                    break

        if self.modelobj is None:
            raise ValueError("Invalid model: %s" % str(self.model))

    def processKnownKey(self, inpkey):
        return self.modelobj.keyScheduleRounds(inpkey, 0, 1)

    def leakage(self, pt, ct, guess, bnum, state):
        try:
            # Make a copy so we don't screw with anything...
            key = list(state['knownkey'])
        except:
            # We don't log due to time-sensitive nature... but if state doesn't have "knownkey" will result in
            # unknown knownkey which causes some attacks to fail. Possibly should make this some sort of
            # flag to indicate we want to ignore the problem?
            key = [None] * 8

        # Guess can be 'none' if we want to use original key as-is
        if guess is not None:
            key[bnum] = guess

        # Get intermediate value
        intermediate_value = self.modelobj.leakage(pt, ct, key, bnum)

        # For bit-wise attacks, mask off specific bit value
        intermediate_value = self._mask & intermediate_value

        # Return HW of guess
        return self.HW[intermediate_value]

    def compare(self, correctKey, guessedKey):
        """Return the bits that are unknown and differ between the guessed key and the correct key"""
        unknown=[]
        wrong=[]
        for i in range(len(correctKey)):
            if guessedKey[i]!=correctKey[i] and i%8 != 7:
                if guessedKey[i] is None:
                    unknown.append(i)
                else:
                    wrong.append(i)
        return (unknown, wrong)

    def keyScheduleRounds(self, inputkey, inputround, desiredround, returnSubkeys=True):
        return self.modelobj.keyScheduleRounds(inputkey, inputround, desiredround, returnSubkeys)


if __name__ == "__main__":
    model = DES()
    # rk = [[0,0,0,1,0,0,1,1,0,0,1,1,0,1,0,0,0,1,0,1,0,1,1,1,0,1,1,1,1,0,0,1,1,0,0,1,1,0,1,1,1,0,1,1,1,1,0,0,1,1,0,1,1,1,1,1,1,1,1,1,0,0,0,1],
    #     [0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0],
    #     [0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 1, 0, 1],
    #     [0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1],
    #     [0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1],
    #     [0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0],
    #     [0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1],
    #     [1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0],
    #     [1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1],
    #     [1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1],
    #     [1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1],
    #     [0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0],
    #     [0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1],
    #     [1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1],
    #     [0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0],
    #     [1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 0],
    #     [1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1]
    # ]
    # for roundNum in range(17):
    #     print model.keyScheduleRounds(rk[0], 0, roundNum)
    #
    # for roundNum in range(0,17):
    #     tmp = model.keyScheduleRounds(rk[roundNum], roundNum, 0)
    #     f=0
    #     for i in range(64):
    #         if tmp[i]!=rk[0][i] and i%8 != 7:
    #             if tmp[i] is None:
    #                 f += 1
    #             else:
    #                 print "Error in bit %d in round %d" % (i, roundNum)
    #     print "%d unrecoverable bits when reverting round key %d" % (f, roundNum)
    #
    # pt = [0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF]
    # for permNum in range(8):
    #     v =  rk[1][permNum*6+0]*32
    #     v +=  rk[1][permNum*6+1]*16
    #     v +=  rk[1][permNum*6+2]*8
    #     v +=  rk[1][permNum*6+3]*4
    #     v +=  rk[1][permNum*6+4]*2
    #     v +=  rk[1][permNum*6+5]*1
    #     print model.sbox_in_first_fbox(pt, v, permNum)

#22 10 30 21 32 38 07 3F
    guessedkey = [0x22, 0x10, 0x30, 0x21, 0x32, 0x38, 0x07, 0x3F]
    originalkey = [0x2B, 0x7E, 0x15, 0x16, 0x28, 0xAE, 0xD2, 0xA6]
    print((str(model.array_of_bytes_to_bin(originalkey, 8))))
    print((str(model.array_of_bytes_to_bin(guessedkey, 6))))

    print(("Unknown bits = %s, Wrong bit guesses = %s" % model.compare(model.array_of_bytes_to_bin(originalkey, 8), model.keyScheduleRounds(guessedkey, 1, 0, returnSubkeys=False))))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2018, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#==========================================================================

# WARNING: This ChipCon implementation is based on a quick-n-dirty port
# of Travis Goodspeed's GOODFET project. It's only designed to be able to perform the
# following: (1) chip erase, (2) read XDATA, (3) read IRAM. This is occasionally useful
# for validating SRAM contents of CC devices.

class ChipConProgrammer(object):

    CMD_CC_PROGRAM = 0x23

    #Goodfet basic commands
    CC_READ  = 0x00
    CC_WRITE = 0x01
    CC_PEEK  = 0x02
    CC_POKE  = 0x03
    CC_SETUP = 0x10
    CC_START = 0x20
    CC_STOP  = 0x21

    #CHIPCON commands
    CC_CHIP_ERASE = 0x80
    CC_WR_CONFIG = 0x81
    CC_RD_CONFIG = 0x82
    CC_GET_PC = 0x83
    CC_READ_STATUS = 0x84
    CC_SET_HW_BRKPNT = 0x85
    CC_HALT = 0x86
    CC_RESUME = 0x87
    CC_DEBUG_INSTR = 0x88
    CC_STEP_INSTR = 0x89
    CC_STEP_REPLACE = 0x8a
    CC_GET_CHIP_ID = 0x8b
    #/CHIPCON macros
    CC_READ_CODE_MEMORY = 0x90
    CC_READ_XDATA_MEMORY = 0x91
    CC_WRITE_XDATA_MEMORY = 0x92
    CC_SET_PC = 0x93
    CC_CLOCK_INIT = 0x94
    CC_WRITE_FLASH_PAGE = 0x95
    CC_READ_FLASH_PAGE = 0x96
    CC_MASS_ERASE_FLASH = 0x97
    CC_PROGRAM_FLASH = 0x98
    CC_WIPEFLASHBUFFER = 0x99
    CC_LOCKCHIP = 0x9A
    #Extra commands (not in original)
    CC_SET_FLASH_WORD_SIZE = 0xA0
    CC_READBACK = 0xA1

    CC_STATUS_ERASED = 0x80
    CC_STATUS_PCONIDLE = 0x40
    CC_STATUS_CPUHALTED = 0x20
    CC_STATUS_PM0 = 0x10
    CC_STATUS_HALTSTATUS = 0x08
    CC_STATUS_LOCKED = 0x04
    CC_STATUS_OSCSTABLE = 0x02
    CC_STATUS_OVERFLOW = 0x01

    def __init__(self, usbdev, timeout=200):
        """
        Set the USB communications instance.
        """

        self._usbdev = usbdev
        self._timeout = timeout

    def _write(self, cmd, arg1=0, data=[]):
        self._usbdev.ctrl_transfer(0x41, self.CMD_CC_PROGRAM, cmd | (arg1 << 8), 0, data, timeout=self._timeout)

    def _read(self, cmd, arg1=0, dlen=1):
        return self._usbdev.ctrl_transfer(0xC1, self.CMD_CC_PROGRAM, cmd | (arg1 << 8), 0, dlen, timeout=self._timeout)


    def enableCC(self, enabled=True):
        if enabled:
            self._write(self.CC_SETUP)
            self._write(self.CC_START)
        else:
            self._write(self.CC_STOP)

    def readSignature(self):
        """Read Signature"""
        cid = self._read(self.CC_GET_CHIP_ID, dlen=2)
        return {'chipid':cid[0], 'rev':cid[1]}

    def eraseChip(self):
        self._write(self.CC_CHIP_ERASE)

    def readXData(self, addr, len=1):
        toread = len

        data = []

        for start in range(0, len, 32):
            ahigh = start & 0xff
            alow = (start >> 8) & 0xff

            if toread < 32:
                dlen = toread
            else:
                dlen = 32

            toread = toread - dlen

            self._write(self.CC_READ_XDATA_MEMORY, data=[dlen, 0, ahigh, alow])
            data.extend(self._read(self.CC_READBACK, dlen=dlen))

        return data

    def readIRam(self, startaddr=0, dlen=1):
        data = []
        for i in range(0, dlen):
            data.append(self._read(self.CC_PEEK, arg1=(startaddr + i), dlen=1)[0])
        return data



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import random
from chipwhisperer.common.utils import util
from ._base import AcqKeyTextPattern_Base
from chipwhisperer.common.utils.util import camel_case_deprecated

class AcqKeyTextPattern_Basic(AcqKeyTextPattern_Base):
    """Class for getting basic keys and plaintexts.

    Basic usage::

        import chipwhisperer as cw
        ktp = cw.ktp.Basic()
        key, text = ktp.next()
    """
    _name = "Basic"

    def __init__(self):
        AcqKeyTextPattern_Base.__init__(self)
        self._fixedKey = True
        self._fixedPlain = False
        self.inittext = '00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F'
        self.initkey = '2B 7E 15 16 28 AE D2 A6 AB F7 15 88 09 CF 4F 3C'
        self._key = util.hexStrToByteArray(self.initkey)
        self._textin = util.hexStrToByteArray(self.inittext)
        self.types = {'Random': False, 'Fixed': True}

    @property
    def fixed_key(self):
        """Generate fixed key (True) or not (False).

        :Getter: Return True if using fixed key or False if not.

        :Setter: Set whether to use fixed key (True) or not (False).

        .. versionadded:: 5.1
            Added fixed_key property
        """
        return self._fixedKey

    @fixed_key.setter
    def fixed_key(self, enabled):
        self._fixedKey = enabled

    @property
    def fixed_text(self):
        """Generate fixed plaintext (True) or not (False).

        :Getter: Return True if using fixed plaintext or False if not.

        :Setter: Set whether to use fixed plaintext (True) or not (False).

        .. versionadded:: 5.1
            Added fixed_text property
        """
        return self._fixedPlain

    @fixed_text.setter
    def fixed_text(self, enabled):
        self._fixedPlain = enabled

    def get_key_type(self):
        return self._fixedKey

    getKeyType = camel_case_deprecated(get_key_type)

    def set_key_type(self, t):
        self._fixedKey = t

    setKeyType = camel_case_deprecated(set_key_type)


    def getPlainType(self):
        return self._fixedPlain

    def setPlainType(self, t):
        self._fixedPlain = t

    def getInitialKey(self):
        return " ".join(["%02X"%b for b in self._key])

    def setInitialKey(self, initialKey, binaryKey=False):
        if initialKey:
            if binaryKey:
                keyStr = ''
                for s in initialKey:
                    keyStr += '%02x ' % s
                self._key = bytearray(initialKey)
            else:
                keyStr = initialKey
                self._key = util.hexStrToByteArray(initialKey)

            self.initkey = keyStr

    def getInitialText(self):
        return " ".join(["%02X" % b for b in self._textin])

    def setInitialText(self, initialText, binaryText=False):
        if initialText:
            if binaryText:
                textStr = ''
                for s in initialText:
                    textStr += '%02x ' % s
                self._textin = bytearray(initialText)
            else:
                textStr = initialText
                self._textin = util.hexStrToByteArray(initialText)

            self.inittext = textStr

    def initPair(self, maxtraces):
        pass

    def new_pair(self):
        if self._fixedKey is False:
            self._key = bytearray(self.keyLen())
            for i in range(0, self.keyLen()):
                self._key[i] = random.randint(0, 255)

        if self._fixedPlain is False:
            self._textin = bytearray(self.textLen())
            for i in range(0, self.textLen()):
                self._textin[i] = random.randint(0, 255)

        # Check pair works with target
        self.validateKey()
        self.validateText()

        return self._key, self._textin

    newPair = new_pair

    def next(self):
        """Returns the next key text pair

        Updates last key and text

        Returns:
            (key (bytearray), text (bytearray))

        .. versionadded:: 5.1
            Added next
        """

        return self.next_key(), self.next_text()

    def next_text(self):
        """ Returns the next plaintext

        Does not update key. If text is fixed, returns the same plaintext as
        last time

        Returns:
            text (bytearray)

        .. versionadded:: 5.1
            Added next_text
        """
        if self._fixedPlain is False:
            self._textin = bytearray(self.textLen())
            for i in range(0, self.textLen()):
                self._textin[i] = random.randint(0, 255)

        self.validateText()
        return self._textin

    def next_key(self):
        """ Returns the next key

        Does not update text. If key is fixed, returns the same key as last
        time

        Returns:
            key (bytearray)

        .. versionadded:: 5.1
            Added next_key
        """
        if self._fixedKey is False:
            self._key = bytearray(self.keyLen())
            for i in range(0, self.keyLen()):
                self._key[i] = random.randint(0, 255)
        self.validateKey()
        return self._key

    """def __str__(self):
        key = "Key=" + self.findParam("Key").getValueKey()
        if self._fixedKey:
            key = key + ":" + self.findParam("initkey").getValue()
        plaintext = "Plaintext=" + self.findParam("Plaintext").getValueKey()
        if self._fixedPlain:
            plaintext = plaintext + ":" + self.findParam("inittext").getValue()

        return self.getName() + " (%s, %s)" % (key, plaintext)"""



#
# SAMx Programming via Python, Copyright (c) 2016-2018 NewAE Technology Inc.
#
# Originally only supported SAM3U, now extended to other devices.
#
# This file is a based on BOSSA, which is:
# Copyright (c) 2011-2012, ShumaTech
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

#Avoid namespace collision with 'serial'


import logging

import serial
import time

class Samba(object):

    def con(self, port, usbmode=True):
        ser = serial.Serial(
            port=port,
            baudrate=921600,  # 115200
            parity=serial.PARITY_NONE,
            stopbits=serial.STOPBITS_ONE,
            bytesize=serial.EIGHTBITS
        )

        self.usbmode = usbmode

        self.ser = ser

        # Auto-baud
        if not usbmode:
            ser.write("\x80")
            ser.flush()
            ser.write("\x80")
            ser.flush()
            ser.write("#")
            ser.flush()
            ser.read(3)

        # Binary mode
        ser.write("N#".encode("ascii"))
        ser.read(2)

        cid = self.chip_id()

        logging.info('FWUP: CID = %04x' % cid)

        #Originally this was used to limit to SAM3U
        #eproc = (cid >> 5) & 0x7
        #arch = (cid >> 20) & 0xff
        #if eproc == 3 and ((0x80 <= arch <= 0x8a) or (0x93 <= arch <= 0x9a)):
        #    logging.info('FWUP: Detected SAM3')

        self.flash = self.get_flash_instance(cid)

        logging.info('FWUP: Detected ' + self.flash.name)
        return True



    def get_flash_instance(self, chipid):

        chipid = chipid & 0x7fffffe0

        #Find settings by looking at following sources:
        # https://github.com/shumatech/BOSSA/blob/master/src/Device.cpp which has EXACT same parameters passed as here
        # OpenOCD files such as https://github.com/ntfreak/openocd/blob/master/src/flash/nor/at91sam4.c

        if chipid == 0x28000960 or chipid == 0x28100960:
            flash = EefcFlash(self, "ATSAM3U4", 0xE0000, 1024, 256, 2, 32, 0x20001000, 0x20008000, 0x400e0800, False)
        elif chipid == 0x280a0760 or chipid == 0x281a0760:
            flash = EefcFlash(self, "ATSAM3U2", 0x80000, 512, 256, 1, 16, 0x20001000, 0x20004000, 0x400e0800, False)
        elif chipid == 0x28090560 or chipid == 0x28190560:
            flash = EefcFlash(self, "ATSAM3U1", 0x80000, 256, 256, 1, 8, 0x20001000, 0x20002000, 0x400e0800, False)
        elif chipid == 0x29970ce0:
            flash = EefcFlash(self, "at91sam4sd16b", 0x400000, 2048, 512, 2, 256, 0x20001000, 0x20010000, 0x400e0a00, False);
        else:
            raise AttributeError("FWUP: Unsupported ChipID = %x" % chipid)

        return flash


    def chip_id(self):
        """ Read chip-id """

        vector = self.read_word(0x00)

        # If the vector is a ARM7TDMI branch, then assume Atmel SAM7 registers
        if ((vector & 0xff000000) == 0xea000000):
            cid = self.read_word(0xfffff240)
        # Else use the Atmel SAM3 registers
        else:
            cid = self.read_word(0x400e0740)

        return cid

    def read_word(self, addr):
        """ Read a word from SAM3U """

        cmd = "w%08X,4#" % addr
        self.ser.write(cmd.encode("ascii"))
        resp = self.ser.read(4)

        value = (resp[3] << 24 | resp[2] << 16 | resp[1] << 8 | resp[0] << 0)
        return value

    def write_word(self, addr, value):
        """ Write a word to SAM3U """

        cmd = "W%08X,%08X#" % (addr, value)
        self.ser.write(cmd.encode("ascii"))

    def read_byte(self, addr):
        """ Read a byte from SAM3U """

        cmd = "o%08X,4#" % addr
        self.ser.write(cmd.encode("ascii"))
        resp = self.ser.read(1)
        return resp[0]

    def go(self, addr):
        """ Run applet """

        cmd = "G%08X#" % addr
        self.ser.write(cmd.encode("ascii"))

        if self.usbmode:
            self.flush()

    def flush(self):
        self.ser.flush()
        # #TODO - if required can add delay
        # #to enforce USB flushing, as happens
        # #after 1mS
        # time.sleep(0.001)


    def _write_buf(self, addr, buf, size):
        """ Write a buffer """

        if self.usbmode == False:
            raise AttributeError("Only USB Mode Supported")

        if len(buf) != size:
            raise AttributeError("Buffer length not as reported, expected {} got {}", size, len(buf))

        self.ser.write(("S%08X,%08X#" % (addr, size)).encode("ascii"))
        # Flush to ensure transactions arrive separately to bootloader
        # (Otherwise error)
        self.flush()
        bwritten = self.ser.write(bytearray(buf))

        if (bwritten != len(buf)):
            raise IOError("Failed to write %d bytes, only %d written" % (len(buf), bwritten))

    def _read_buf(self, addr, size):
        """ Read a buffer """

        if self.usbmode == False:
            raise AttributeError("Only USB Mode Supported")

        buf = []

        # Note from BOSSA:
        #   The SAM firmware has a bug reading powers of 2 over 32 bytes
        #   via USB.  If that is the case here, then read the first byte
        #   with a readByte and then read one less than the requested size.
        if self.usbmode and (size > 32) and not (size & (size - 1)):
            buf.append(self.read_byte(addr))
            addr = addr + 1
            size = size - 1

        cmd = "R%08X,%08X#" % (addr, size)
        self.ser.write(cmd.encode("ascii"))
        buf.extend(self.ser.read(size))
        return buf


    def erase(self):
        """ Erase entire chip """

        self.flash.eraseAll()
        self.flash.eraseAuto(False)

    def write(self, bindata, doprint=False):
        """ Write a buffer into chip """

        page_size = self.flash.size
        page_num = 0
        page_max = self.flash.pages

        totalpages = (len(bindata) + page_size - 1) / page_size
        if totalpages > page_max:
            raise IOError("File too big (need %d pages, have %d)" % (totalpages, page_max))

        bytesleft = len(bindata)

        i = 0
        while bytesleft > 0:

            if bytesleft < page_size:
                # Pad partial page with 0xFF
                buf = bytearray(bindata[i:(i + bytesleft)])
                # print(len(buf))
                buf.extend([0xff] * (page_size - bytesleft))
                # print(bytesleft)
            else:
                # Read full page
                buf = bindata[i:(i + page_size)]

            if (page_num % 10) == 0 and doprint:
                logging.debug('Flashing %d/%d' % (page_num, totalpages))

            self.flash.loadBuffer(buf)
            self.flash.writePage(page_num)

            page_num += 1
            if (page_num == totalpages) or (len(buf) != page_size):
                break

            i += page_size
            bytesleft -= page_size

        logging.info('FWUP: Program Successful')

    def verify(self, bindata, doprint=False):
        """ Verify a buffer that was written into chip """

        page_size = self.flash.size
        page_num = 0
        page_max = self.flash.pages

        totalpages = (len(bindata) + page_size - 1) / page_size
        if totalpages > page_max:
            raise IOError("File too big (need %d pages, have %d)" % (totalpages, page_max))

        bytesleft = len(bindata)

        i = 0
        while bytesleft > 0:

            if bytesleft < page_size:
                # Pad partial page with 0xFF
                buf = bytearray(bindata[i:(i + bytesleft)])
                buf.extend([0xff] * (page_size - bytesleft))
            else:
                # Read full page
                buf = bindata[i:(i + page_size)]

            if (page_num % 10) == 0 and doprint:
                print('Verifying %d/%d' % (page_num, totalpages))

            bufferB = bytearray(self.flash.readPage(page_num))

            if bytearray(buf) != bytearray(bufferB):
                # logging.warning('FWUP: Verify FAILED at %d"=' % i)
                logging.warning("Verify failed at {} (got {} expected {})".format(i, buf, bufferB))
                return False
                # print "fail at %d"%i
                # print "".join(["%02x"%ord(a) for a in buf])

            page_num = page_num + 1
            if (page_num == totalpages) or (len(buf) != page_size):
                break

            i = i + page_size
            bytesleft = bytesleft - page_size

        logging.info('FWUP: Verify successful')

        return True


class WordCopyArm(object):
    _dst_addr = 0x00000028
    _src_addr = 0x0000002c
    _reset = 0x00000024
    _stack = 0x00000020
    _start = 0x00000000
    words = 0x00000030
    code = [0x09, 0x48, 0x0a, 0x49, 0x0a, 0x4a, 0x02, 0xe0, 0x08, 0xc9, 0x08, 0xc0, 0x01, 0x3a, 0x00, 0x2a,
            0xfa, 0xd1, 0x04, 0x48, 0x00, 0x28, 0x01, 0xd1, 0x01, 0x48, 0x85, 0x46, 0x70, 0x47, 0xc0, 0x46,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00]


class WordCopyApplet(object):
    def __init__(self, samba, addr):
        self._samba = samba
        self._addr = addr
        self._applet = WordCopyArm()
        self._samba._write_buf(addr, self._applet.code, len(self._applet.code))

    def set_dst_addr(self, addr):
        self._samba.write_word(self._addr + self._applet._dst_addr, addr)

    def set_src_addr(self, addr):
        self._samba.write_word(self._addr + self._applet._src_addr, addr)

    def set_words(self, words):
        self._samba.write_word(self._addr + self._applet.words, words)

    def set_stack(self, stack):
        self._samba.write_word(self._addr + self._applet._stack, stack)

    def run(self):
        self._samba.go(self._addr + self._applet._start + 1)

    def runv(self):
        self._samba.write_word(self._addr + self._applet._reset, self._addr + self._applet._start + 1)
        self._samba.go(self._addr + self._applet._stack)

    def size(self):
        return len(self._applet.code)

    def addr(self):
        return self._addr


class EefcFlash(object):
    def __init__(self, samba, name, addr, pages, size, planes, lock_regions, user, stack, regs, can_brownout):
        self.name = name
        self.addr = addr
        self.pages = pages
        self.size = size
        self.planes = planes
        self.lock_regions = lock_regions
        self.user = user
        self.stack = stack
        self.regs = regs
        self.can_brownout = can_brownout
        self.erase_auto = True
        self.samba = samba
        self.word_copy = WordCopyApplet(samba, user)

        self.EEFC_KEY = 0x5a

        self.EEFC0_FMR = (regs + 0x00)
        self.EEFC0_FCR = (regs + 0x04)
        self.EEFC0_FSR = (regs + 0x08)
        self.EEFC0_FRR = (regs + 0x0C)

        self.EEFC1_FMR = (regs + 0x200)
        self.EEFC1_FCR = (regs + 0x204)
        self.EEFC1_FSR = (regs + 0x208)
        self.EEFC1_FRR = (regs + 0x20C)

        self.EEFC_FCMD_GETD = 0x0
        self.EEFC_FCMD_WP = 0x1
        self.EEFC_FCMD_WPL = 0x2
        self.EEFC_FCMD_EWP = 0x3
        self.EEFC_FCMD_EWPL = 0x4
        self.EEFC_FCMD_EA = 0x5
        self.EEFC_FCMD_SLB = 0x8
        self.EEFC_FCMD_CLB = 0x9
        self.EEFC_FCMD_GLB = 0xa
        self.EEFC_FCMD_SGPB = 0xb
        self.EEFC_FCMD_CGPB = 0xc
        self.EEFC_FCMD_GGPB = 0xd

        self.word_copy.set_words(int(size / 4))
        self.word_copy.set_stack(stack)
        self._onBufferA = True
        self._pageBufferA = self.user + self.word_copy.size()
        self._pageBufferB = self._pageBufferA + size

        # SAM3 Errata (FWS must be 6)
        self.samba.write_word(self.EEFC0_FMR, 0x6 << 8)
        if (planes == 2):
            self.samba.write_word(self.EEFC1_FMR, 0x6 << 8)


    def eraseAll(self):
        self.waitFSR()
        self.writeFCR0(self.EEFC_FCMD_EA, 0)
        if (self.planes == 2):
            self.waitFSR()
            self.writeFCR1(self.EEFC_FCMD_EA, 0)

    def lockAll(self):
        for region in range(0, self.lock_regions):
            self.setLockRegion(region, True)

    def unlockAll(self):
        for region in range(0, self.lock_regions):
            self.setLockRegion(region, False)

    def loadBuffer(self, data):
        if self._onBufferA:
            self.samba._write_buf(self._pageBufferA, data, self.size)
        else:
            self.samba._write_buf(self._pageBufferB, data, self.size)

    def eraseAuto(self, enable):
        self.erase_auto = enable

    def isLocked(self):
        self.waitFSR()
        self.writeFCR0(self.EEFC_FCMD_GLB, 0)
        self.waitFSR()
        if (self.readFRR0()):
            return True
        if (self.planes == 2):
            self.writeFCR1(self.EEFC_FCMD_GLB, 0)
            self.waitFSR()
            if (self.readFRR1()):
                return True
        return False

    def getLockRegion(self, region):
        if (region >= self.lockRegions):
            raise AttributeError("Invalid flash region")

        self.waitFSR()

        if (self.planes == 2 and region >= self.lockRegions / 2):
            self.writeFCR1(self.EEFC_FCMD_GLB, 0)
            self.waitFSR()
            if self.readFRR1() & (1 << (region - (self.lock_regions / 2))):
                return True
        else:
            self.writeFCR0(self.EEFC_FCMD_GLB, 0)
            self.waitFSR()
            if self.readFRR0() & (1 << region):
                return True

        return False


    def setLockRegion(self, region, enable):

        if (region >= self.lockRegions):
            raise AttributeError("Invalid flash region")

        if (enable != self.getLockRegion(region)):
            if (self.planes == 2) and (region >= self.lockRegions / 2):
                page = (region - self.lockRegions / 2) * self.pages / self.lockRegions;
                self.waitFSR()
                if enable:
                    self.writeFCR1(self.EEFC_FCMD_SLB, page)
                else:
                    self.writeFCR1(self.EEFC_FCMD_CLB, page)
            else:
                page = int(region * self.pages / self.lockRegions)
                self.waitFSR()
                if enable:
                    self.writeFCR0(self.EEFC_FCMD_SLB, page)
                else:
                    self.writeFCR0(self.EEFC_FCMD_CLB, page)

    def getSecurity(self):
        self.waitFSR()
        self.writeFCR0(self.EEFC_FCMD_GGPB, 0)
        self.waitFSR()
        return (self.readFRR0() & (1 << 0))

    def setSecurity(self):
        self.waitFSR()
        self.writeFCR0(self.EEFC_FCMD_SGPB, 0)

    def getBod(self):
        if not self.can_brownout:
            return False

        self.waitFSR()
        self.writeFCR0(self.EEFC_FCMD_GGPB, 0)
        self.waitFSR()
        return (self.readFRR0() & (1 << 1))

    def setBod(self, enable):
        if not self.can_brownout:
            return

        self.waitFSR()
        if enable:
            self.writeFCR0(self.EEFC_FCMD_SGPB, 1)
        else:
            self.writeFCR0(self.EEFC_FCMD_CGPB, 1)

    def getBor(self):
        if not self.can_brownout:
            return False

        self.waitFSR()
        self.writeFCR0(self.EEFC_FCMD_GGPB, 0)
        self.waitFSR()
        return (self.readFRR0() & (1 << 2))

    def setBor(self, enable):
        if not self.can_brownout:
            return

        self.waitFSR()
        if enable:
            self.writeFCR0(self.EEFC_FCMD_SGPB, 2)
        else:
            self.writeFCR0(self.EEFC_FCMD_CGPB, 2)

    def getBootFlash(self):
        self.waitFSR()
        self.writeFCR0(self.EEFC_FCMD_GGPB, 0)
        self.waitFSR()
        if self.can_brownout:
            cb = 3
        else:
            cb = 1
        return (self.readFRR0() & (1 << cb))

    def setBootFlash(self, enable):
        self.waitFSR()
        if self.can_brownout:
            cb = 3
        else:
            cb = 1

        if enable:
            self.writeFCR0(self.EEFC_FCMD_SGPB, cb)
        else:
            self.writeFCR0(self.EEFC_FCMD_CGPB, cb)

    def writePage(self, page):
        if (page >= self.pages):
            raise AttributeError("Invalid Page")

        self.word_copy.set_dst_addr(self.addr + (page * self.size))
        if self._onBufferA:
            self.word_copy.set_src_addr(self._pageBufferA)
        else:
            self.word_copy.set_src_addr(self._pageBufferB)
        self._onBufferA = not self._onBufferA
        self.waitFSR()
        self.word_copy.runv()

        if self.erase_auto:
            cmd = self.EEFC_FCMD_EWP
        else:
            cmd = self.EEFC_FCMD_WP

        if (self.planes == 2 and page >= (self.pages / 2)):
            self.writeFCR1(cmd, page - self.pages / 2)
        else:
            self.writeFCR0(cmd, page)

    def readPage(self, page):
        if (page >= self.pages):
            raise AttributeError("Invalid page")

        # Note from BOSSA:
        #  The SAM3 firmware has a bug where it returns all zeros for reads
        #  directly from the flash so instead, we copy the flash page to
        #  SRAM and read it from there.

        if self._onBufferA:
            self.word_copy.set_dst_addr(self._pageBufferA)
        else:
            self.word_copy.set_dst_addr(self._pageBufferB)

        self.word_copy.set_src_addr(self.addr + (page * self.size))
        self.waitFSR()
        self.word_copy.runv()
        if self._onBufferA:
            return self.samba._read_buf(self._pageBufferA, self.size)
        else:
            return self.samba._read_buf(self._pageBufferB, self.size)


    def waitFSR(self, wait_ms=5000):
        tries = 0
        fsr1 = 0x1

        while (tries <= wait_ms):
            tries = tries + 1
            fsr0 = self.samba.read_word(self.EEFC0_FSR)
            if (fsr0 & (1 << 2)):
                raise IOError("Timeout")

            if (self.planes == 2):
                fsr1 = self.samba.read_word(self.EEFC1_FSR)
                if (fsr1 & (1 << 2)):
                    raise IOError("Timeout")
            if (fsr0 & fsr1 & 0x1):
                break;
            time.sleep(0.001)
        if (tries > wait_ms):
            raise IOError("Timeout")

    def writeFCR0(self, cmd, arg):
        self.samba.write_word(self.EEFC0_FCR, (self.EEFC_KEY << 24) | (arg << 8) | cmd)

    def writeFCR1(self, cmd, arg):
        self.samba.write_word(self.EEFC1_FCR, (self.EEFC_KEY << 24) | (arg << 8) | cmd)

    def readFRR0(self):
        return self.samba.read_word(self.EEFC0_FRR)

    def readFRR1(self):
        return self.samba.read_word(self.EEFC1_FRR)

if __name__ == "__main__":
    # Example usage
    logging.basicConfig(level=logging.INFO)
    sam = Samba()
    sam.con('com131')
    sam.erase()
    #data = open(r'C:\chipwhisperer\hardware\capture\chipwhisperer-lite\sam3u_fw\SAM3U_VendorExample\Debug\SAM3U_CW1173.bin', 'rb').read()
    data = open(r'c:\users\colin\dropbox\engineering\git_repos\CW1101_CWNano\cwnano-firmware\cwnano-firmware\Debug\cwnano-firmware.bin', 'rb').read()
    sam.write(data)
    if sam.verify(data):
        sam.flash.setBootFlash(True)
    sam.ser.close()




from chipwhisperer.analyzer.attacks.snr import calculate_snr
from chipwhisperer.analyzer.attacks import cpa_algorithms
from chipwhisperer.analyzer import preprocessing
from chipwhisperer.common.utils.util import camel_case_deprecated
from chipwhisperer.common.api.ProjectFormat import Project
from chipwhisperer.analyzer.utils import aes_funcs as aes_funcs
from chipwhisperer.analyzer.attacks.models import EightBitAES128LeakageModels
from chipwhisperer.analyzer.attacks.models.AES128_8bit import AESLeakageHelper


leakage_models = EightBitAES128LeakageModels()


def cpa(proj, leak_model, algorithm=cpa_algorithms.Progressive):
    """Create a CPA object to attack traces in project using leak_model and algorithm.

    Args:
       proj (Project): TraceManager or preprocessing object that
           holds traces, plaintext, etc.
       leak_model (AESLeakageHelper): Model describing information
           leaked from target.
       algorithm (AlgorithmsBase, optional): Algorithm to perform CPA loop.
           Default is almost always the right choice. Defaults to
           cpa_algorithms.Progressive.

    Returns:
       A CPA object. For more information, see documentation for CPA.

    .. versionchanged:: 5.1
        Now uses new CPA attack object
    """
    from chipwhisperer.analyzer.attacks.cpa_new import CPA
    attack = CPA(proj, leak_model, algorithm)

    return attack


def profiling(trace_source):
    """Not yet implemented, create/import manually"""
    raise NotImplementedError


def analyzer_plots(attack_results=None):
    """Create an object to get plot data for analyzer results

    Args:
        attack_results(Results, optional): Results from attack to use
            to generate plots.

    Returns:
        Object to generate plots.
    """
    from chipwhisperer.common.results.noguiplots import NoGUIPlots
    return NoGUIPlots(attack_results)

analyzerPlots = analyzer_plots

# JUPYTER ONLY
# current_trace_iteration = 0
current_trace_iteration = 0
def _default_jupyter_callback(attack, head = 6, fmt="{:02X}<br>{:.3f}"):
    import pandas as pd
    from IPython.display import clear_output
    global current_trace_iteration
    attack_results = attack.results
    key = attack.project.keys[0]

    def format_stat(stat):
        if type(stat) is int:
            return str(stat)
        return str(fmt.format(stat[0], stat[2]))

    def color_corr_key(row):
        ret = [""] * 16
        key = attack.known_key()
        for i,bnum in enumerate(row):
            if type(bnum) is int:
                continue
            if bnum[0] == key[i]:
                ret[i] = "color: red"
            else:
                ret[i] = ""
        return ret
    attack_results.set_known_key(key)
    stat_data = attack_results.find_maximums()
    df = pd.DataFrame(stat_data).transpose()

    #Add PGE row
    df_pge = pd.DataFrame(attack_results.pge).transpose().rename(index={0:"PGE="}, columns=int)
    df = pd.concat([df_pge, df], ignore_index=False)

    clear_output(wait=True)
    reporting_interval = attack.get_reporting_interval()
    tstart = current_trace_iteration * reporting_interval
    tend = tstart + reporting_interval
    current_trace_iteration += 1
    display(df.head(head).style.format(format_stat).apply(color_corr_key, axis=1).set_caption("Finished traces {} to {}".format(tstart, tend)))


def get_jupyter_callback(attack, head = 6, fmt="{:02X}<br>{:.3f}"):
    """Get callback for use in Jupyter"""
    global current_trace_iteration
    current_trace_iteration = 0
    return lambda : _default_jupyter_callback(attack, head, fmt)
getJupyterCallback = camel_case_deprecated(get_jupyter_callback)


def reset_iteration():
    global current_trace_iteration
    current_trace_iteration = 0







#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='chipwhisperer',
    version='5.1.1',
    description="ChipWhisperer Side-Channel Analysis Tool",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author="Colin O'Flynn",
    author_email='coflynn@newae.com',
    license='GPLv3',
    url='https://www.chipwhisperer.com',
    packages=find_packages('software'),
    package_dir={'': 'software'},
    install_requires=[
        'configobj',
        'pyserial',
        'docutils',  # makes help more readable.
        'numpy',
        'pyusb',
        'scipy',
        'pycryptodome'
    ],
    project_urls={
        'Documentation': 'https://chipwhisperer.readthedocs.io',
        'Source': 'https://github.com/newaetech/chipwhisperer',
        'Issue Tracker': 'https://github.com/newaetech/chipwhisperer/issues',
    },
    python_requires='~=3.4',
)



#!/usr/bin/python
# HIGHLEVEL_CLASSLOAD_FAIL_FUNC_WARN
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging
from usb import USBError
from .cwhardware import ChipWhispererDecodeTrigger, ChipWhispererDigitalPattern, ChipWhispererExtra, ChipWhispererSAD
from . import _qt as openadc_qt
from .base import ScopeTemplate
from .openadc_interface.naeusbchip import OpenADCInterface_NAEUSBChip
from chipwhisperer.common.utils import util
from chipwhisperer.common.utils.util import dict_to_str
from collections import OrderedDict
import time

class OpenADC(ScopeTemplate, util.DisableNewAttr):
    """OpenADC scope object.

    This class contains the public API for the OpenADC hardware, including the
    ChipWhisperer Lite/ CW1200 Pro boards. It includes specific settings for
    each of these devices.

    To connect to one of these devices, the easiest method is::

        import chipwhisperer as cw
        scope = cw.scope(type=scopes.OpenADC)

    Some sane default settings are available via::

        scope.default_setup()

    This code will automatically detect an attached ChipWhisperer device and
    connect to it.

    For more help about scope settings, try help() on each of the ChipWhisperer
    scope submodules (scope.gain, scope.adc, scope.clock, scope.io,
    scope.trigger, and scope.glitch):

     *  :attr:`scope.gain <.OpenADC.gain>`
     *  :attr:`scope.adc <.OpenADC.adc>`
     *  :attr:`scope.clock <.OpenADC.clock>`
     *  :attr:`scope.io <.OpenADC.io>`
     *  :attr:`scope.trigger <.OpenADC.trigger>`
     *  :attr:`scope.glitch <.OpenADC.glitch>`
     *  :meth:`scope.default_setup <.OpenADC.default_setup>`
     *  :meth:`scope.con <.OpenADC.con>`
     *  :meth:`scope.dis <.OpenADC.dis>`
     *  :meth:`scope.arm <.OpenADC.arm>`
     *  :meth:`scope.get_last_trace <.OpenADC.get_last_trace>`

    If you have a CW1200 ChipWhisperer Pro, you have access to some additional features:

     * :attr:`scope.SAD <.OpenADC.SAD>`
     * :attr:`scope.DecodeIO <.OpenADC.DecodeIO>`
     * :attr:`scope.adc.stream_mode (see scope.adc for more information)`
    """

    _name = "ChipWhisperer/OpenADC"

    def __init__(self):
        ScopeTemplate.__init__(self)

        self.qtadc = openadc_qt.OpenADCQt()
        self.qtadc.dataUpdated.connect(self.newDataReceived)
        # Bonus Modules for ChipWhisperer
        self.advancedSettings = None
        self.advancedSAD = None
        self.digitalPattern = None

        self._is_connected = False

        self.scopetype = OpenADCInterface_NAEUSBChip(self.qtadc)

    def _getNAEUSB(self):
        return self.scopetype.dev._cwusb

    def default_setup(self):
        """Sets up sane capture defaults for this scope

         *  45dB gain
         *  5000 capture samples
         *  0 sample offset
         *  rising edge trigger
         *  7.37MHz clock output on hs2
         *  4*7.37MHz ADC clock
         *  tio1 = serial rx
         *  tio2 = serial tx

        .. versionadded:: 5.1
            Added default setup for OpenADC
        """
        self.gain.db = 25
        self.adc.samples = 5000
        self.adc.offset = 0
        self.adc.basic_mode = "rising_edge"
        self.clock.clkgen_freq = 7.37e6
        self.trigger.triggers = "tio4"
        self.io.tio1 = "serial_rx"
        self.io.tio2 = "serial_tx"
        self.io.hs2 = "clkgen"

        self.clock.adc_src = "clkgen_x4"

        count = 0
        while not self.clock.clkgen_locked:            
            self.clock.reset_dcms()
            time.sleep(0.05)
            count += 1

            if count == 5:
                logging.info("Could not lock clock for scope. This is typically safe to ignore. Reconnecting and retrying...")
                self.dis()
                time.sleep(0.25)
                self.con()
                time.sleep(0.25)
                self.gain.db = 25
                self.adc.samples = 5000
                self.adc.offset = 0
                self.adc.basic_mode = "rising_edge"
                self.clock.clkgen_freq = 7.37e6
                self.trigger.triggers = "tio4"
                self.io.tio1 = "serial_rx"
                self.io.tio2 = "serial_tx"
                self.io.hs2 = "clkgen"
                self.clock.adc_src = "clkgen_x4"

            if count > 10:
                raise OSError("Could not lock DCM. Try rerunning this function or calling scope.clock.reset_dcms(): {}".format(self))
    def dcmTimeout(self):
        if self.connectStatus:
            try:
                self.qtadc.sc.getStatus()
            except USBError:
                self.dis()
                raise Warning("Error in the scope. It may have been disconnected.")
            except Exception as e:
                self.dis()
                raise e

    def getCurrentScope(self):
        return self.scopetype

    def setCurrentScope(self, scope):
        self.scopetype = scope

    def _getCWType(self):
        """Find out which type of ChipWhisperer this device is.

        Returns:
            One of the following:
             -  ""
             -  "cwlite"
             -  "cw1200"
             -  "cwrev2"
        """
        hwInfoVer = self.qtadc.sc.hwInfo.versions()[2]
        if "ChipWhisperer" in hwInfoVer:
            if "Lite" in hwInfoVer:
                return "cwlite"
            elif "CW1200" in hwInfoVer:
                return "cw1200"
            else:
                return "cwrev2"
        return ""

    def get_name(self):
        """ Gets the name of the attached scope

        Returns:
            'ChipWhisperer Lite' if a Lite, 'ChipWhisperer Pro' if a Pro
        """
        name = self._getCWType()
        if name == "cwlite":
            return "ChipWhisperer Lite"
        elif name == "cw1200":
            return "ChipWhisperer Pro"

    def _con(self, sn=None):
        if self.scopetype is not None:
            self.scopetype.con(sn)

            self.qtadc.sc.usbcon = self.scopetype.ser._usbdev

            cwtype = self._getCWType()
            if cwtype != "":
                self.advancedSettings = ChipWhispererExtra.ChipWhispererExtra(cwtype, self.scopetype, self.qtadc.sc)

                util.chipwhisperer_extra = self.advancedSettings

                if cwtype == "cwrev2" or cwtype == "cw1200":
                    self.SAD = ChipWhispererSAD.ChipWhispererSAD(self.qtadc.sc)

                if cwtype == "cw1200":
                    self.decode_IO = ChipWhispererDecodeTrigger.ChipWhispererDecodeTrigger(self.qtadc.sc)
                    #self.advancedSettings.cwEXTRA.triggermux._set_is_pro(True)

                if cwtype == "cwcrev2":
                    self.digitalPattern = ChipWhispererDigitalPattern.ChipWhispererDigitalPattern(self.qtadc.sc)

            self.adc = self.qtadc.parm_trigger
            self.gain = self.qtadc.parm_gain
            self.clock = self.qtadc.parm_clock

            if cwtype == "cw1200":
                self.adc._is_pro = True
            if self.advancedSettings:
                self.io = self.advancedSettings.cwEXTRA.gpiomux
                self.trigger = self.advancedSettings.cwEXTRA.triggermux
                self.glitch = self.advancedSettings.glitch.glitchSettings
                if cwtype == "cw1200":
                    self.trigger = self.advancedSettings.cwEXTRA.protrigger


            self.disable_newattr()
            self._is_connected = True
            return True
        return False

    def _dis(self):
        if self.scopetype is not None:
            self.scopetype.dis()
            if self.advancedSettings is not None:
                self.advancedSettings = None
                util.chipwhisperer_extra = None

            if self.advancedSAD is not None:
                self.advancedSAD = None

            if self.digitalPattern is not None:
                self.digitalPattern = None

        # TODO Fix this hack
        if hasattr(self.scopetype, "ser") and hasattr(self.scopetype.ser, "_usbdev"):
            self.qtadc.sc.usbcon = None

        self.enable_newattr()
        self._is_connected = False
        return True

    def arm(self):
        """Setup scope to begin capture/glitching when triggered.

        The scope must be armed before capture or glitching (when set to
        'ext_single') can begin.

        Raises:
           OSError: Scope isn't connected.
           Exception: Error when arming. This method catches these and
               disconnects before reraising them.
        """
        if self.connectStatus is False:
            raise OSError("Scope is not connected. Connect it first...")

        try:
            if self.advancedSettings:
                self.advancedSettings.armPreScope()

            self.qtadc.arm()

            if self.advancedSettings:
                 self.advancedSettings.armPostScope()

            self.qtadc.startCaptureThread()
        except Exception:
            self.dis()
            raise

    def capture(self):
        """Captures trace. Scope must be armed before capturing.

        Returns:
           True if capture timed out, false if it didn't.

        Raises:
           IOError: Unknown failure.
        """
        if not self.adc.stream_mode:
            return self.qtadc.capture(self.adc.offset)
        else:
            return self.qtadc.capture(None)
        return ret

    def get_last_trace(self):
        """Return the last trace captured with this scope.

        Returns:
           Numpy array of the last capture trace.
        """
        return self.qtadc.datapoints

    getLastTrace = util.camel_case_deprecated(get_last_trace)

    def _dict_repr(self):
        dict = OrderedDict()
        dict['gain']    = self.gain._dict_repr()
        dict['adc']     = self.adc._dict_repr()
        dict['clock']   = self.clock._dict_repr()
        dict['trigger'] = self.trigger._dict_repr()
        dict['io']      = self.io._dict_repr()
        dict['glitch']  = self.glitch._dict_repr()
        if self._getCWType() == "cw1200":
            dict['SAD'] = self.SAD._dict_repr()
            dict['decode_IO'] = self.decode_IO._dict_repr()

        return dict

    def __repr__(self):
        # Add some extra information about ChipWhisperer type here
        if self._is_connected:
            ret = "%s Device\n" % self._getCWType()
            return ret + dict_to_str(self._dict_repr())
        else:
            ret = "ChipWhisperer/OpenADC device (disconnected)"
            return ret

    def __str__(self):
        return self.__repr__()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import numpy as np
import os
import sys
from operator import itemgetter


class ChannelEstimateAttackOneSubkey(object):
    """ Redos entire attack """

    def __init__(self):        
        self.clearStats()
        
    def clearStats(self):
        pass
    
    def oneSubkey(self, bnum, traces_fit, traces_test, plaintexts_fit, plaintexts_test,
                  model, modeltype="Hamming Weight",
                  key=None, printData=False, aroundStartEnd=False, useSVD=True, tracefitPInv=False,
                  progressBar=None, pbcnt=0):

        if tracefitPInv:
            ntracesfit = len(traces_fit[0,:])
        else:
            ntracesfit = len(traces_fit[:,0])
        ntracestry = len(traces_test[:,0])

        if printData:
            print(" Key %d = %2x"%(bnum, key[bnum]))

        start = 0
        end = 256

        if aroundStartEnd:
            start = key[bnum] - 5
            end = key[bnum] + 6

        if start < 0:
            start = 0
        
        if end > 256:
            end = 256

        result = []

        diffs = [1E99]*256

        for guess in range(start, end):
            if printData:
                print("  %2x "%guess, end=' ')
                
            hwlst = np.zeros( (ntracesfit, 1) )

            for i,t in enumerate(plaintexts_fit):
                hwlst[i][0] = AES128_8bit.HypHW(t, None, guess, bnum)
    
            if useSVD:
                if tracefitPInv:
                    #traces_fit already a pseudoinverse
                    channel = np.dot(traces_fit, hwlst)
                else:
                    #Use pseudo-inverse which is fairly fast, as NumPY uses a SVD decomposition
                    channel = np.dot(np.linalg.pinv(traces_fit), hwlst) 
            else:
                #Classic but VERY SLOW least-squares
                channel = np.linalg.lstsq(traces_fit, hwlst)[0]           

            hwtry = np.zeros( (ntracestry, 1) )
            for i,t in enumerate(plaintexts_test):
                hwtry[i][0] = AES128_8bit.HypHW(t, None, guess, bnum)

            res = np.dot(traces_test,channel)
            res = ((res-hwtry)**2).sum()   

            diffs[guess] = res

            if printData:
                print("%E"%res)
                result.append({'sse':res, 'guess':guess, 'csi':channel})
    
        if printData:        
            sortedresult = sorted(result, key=itemgetter('sse'))
            print("answer @ rank %d"%map(itemgetter('guess'), sortedresult).index(key[bnum]))

        return (diffs, pbcnt)


#Hacks for main()
sys.path.append('../../common')
sys.path.append('../.')
from .models import AES128_8bit as AES128_8bit

def main(tracedir, fitrange, tryrange):    
    preflist = []

    files = os.listdir(tracedir)
    for f in files:
        if "_traces.npy" in f:
            f = f.replace("traces.npy", "")
            preflist.append(f)

    if len(preflist) == 0:
        print("WARNING: No traces found. Check directory.")

    prefix = preflist[0]

    traces = np.load(tracedir + prefix + "traces.npy")
    textin = np.load(tracedir + prefix + "textin.npy")
    key = np.load(tracedir + prefix + "knownkey.npy")
    
    #Majority of traces used in generating estimated channel
    tracesfit = traces[fitrange[0]:fitrange[1],:]
    textinfit = textin[fitrange[0]:fitrange[1],:]

    #Select a few traces to use a test
    tracestry = traces[tryrange[0]:tryrange[1],:]
    textintry = textin[tryrange[0]:tryrange[1],:]

    tracesfit = np.linalg.pinv(tracesfit)

    cea = ChannelEstimateAttackOneSubkey()

    diffs = [0]*16
    blist = list(range(0, 16))

    for bnum in blist:
        print("Attacking byte %d"%bnum)
        cea.clearStats()

        #Example running an attack, don't set 'aroundStartEnd' if you want
        #the full attack and not quick cheater
        diffs[bnum] = cea.oneSubkey(bnum, tracesfit, tracestry, textinfit, textintry, AES128_8bit,
                      key=key, printData=False, aroundStartEnd=False, tracefitPInv=True)[0]

    #Sort Output
    output = [0]*16
    for bnum in blist:
        output[bnum] = []
        for i,d in enumerate(diffs[bnum]):
            output[bnum].append({'hyp':i, 'sse':d})

        output[bnum] = sorted(output[bnum], key=itemgetter('sse'))

    #Print most likely & PGE
    for bnum in blist:
        print(" %02x "%output[bnum][0]['hyp'], end=' ')
    print("")
    for bnum in blist:
        print("%3d "%map(itemgetter('hyp'), output[bnum]).index(key[bnum]), end=' ')
    print("")


if __name__ == "__main__":
    #Point to some example traces, available from ChipWhisperer.com

    #RSM-AES256 (DPAv4 Contest)
    directory = "/home/colin/Documents/chipwhisperer/git/chipwhisperer/software/scripting-examples/default-data-dir/traces-avr-dpav4-vcc/"
    fitrange = (0, 4000)
    tryrange = (4000, 10000)

    #Simple AES
    #directory = "/home/colin/Documents/chipwhisperer/git/chipwhisperer/software/scripting-examples/default-data-dir/traces-avr-aes128-vcc/"
    #fitrange = (0, 100)
    #tryrange = (8000, 8150)

    #Input traces need to be split into a "fitting set" and a "test set"
    #A channel will be generated with the "fitting set", and later we'll
    #try then with the "test set". The two sets are used to avoid fitting
    #random noise, and check how things actually worked
    
    main(directory, fitrange, tryrange)



#
# Copyright (c) 2014-2018, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.chipwhisperer.com
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
# ChipWhisperer is a trademark of NewAE Technology Inc., registered in the
# United States of America, the European Union, and other jurisdictions.
# ==========================================================================
import logging
import warnings
import time
import math
from threading import Condition, Thread
import struct
import pickle
import traceback
import os

from usb.backend import libusb0
import usb.core
import usb.util

from chipwhisperer.hardware.firmware import cwlite as fw_cwlite
from chipwhisperer.hardware.firmware import cw1200 as fw_cw1200
from chipwhisperer.hardware.firmware import cw305  as fw_cw305



def packuint32(data):
    """Converts a 32-bit integer into format expected by USB firmware"""

    data = int(data)
    return [data & 0xff, (data >> 8) & 0xff, (data >> 16) & 0xff, (data >> 24) & 0xff]

def unpackuint32(buf):
    """"Converts an array into a 32-bit integer"""

    pint = buf[0]
    pint |= buf[1] << 8
    pint |= buf[2] << 16
    pint |= buf[3] << 24
    return pint

def packuint16(data):
    """Converts a 16-bit integer into format expected by USB firmware"""

    data = int(data)

    return [data & 0xff, (data >> 8) & 0xff, (data >> 16) & 0xff, (data >> 24) & 0xff]


#List of all NewAE PID's
NEWAE_VID = 0x2B3E
NEWAE_PIDS = {
    0xACE2: {'name': "ChipWhisperer-Lite",     'fwver': fw_cwlite.fwver},
    0xACE3: {'name': "ChipWhisperer-CW1200",   'fwver': fw_cw1200.fwver},
    0xC305: {'name': "CW305 Artix FPGA Board", 'fwver': fw_cw305.fwver},
}

class NAEUSB_Serializer_base(object):
    """
    Base clase for packaging USB commands. The seralizer class is used when the USB may talk via a daemon instead
    of directly running from the Python code. This is useful when you want the USB device to be handled asynchronously,
    such as required to keep serial data being sent to/from the device.
    """

    FLUSH_INPUT = 0xE0
    CLOSE = 0xF0
    OPEN = 0xF1
    WRITE_CTRL = 0xF2
    READ_CTRL = 0xF3
    CMD_WRITE_MEM = 0xF4
    CMD_READ_MEM = 0xF5
    GET_POSSIBLE_DEVICES = 0xF6
    WRITE_BULK = 0xF7
    READ_BULK = 0xF8


    ACK = 0xA0
    ERROR = 0xA1

    def make_cmd(self, cmd, datalist):

        if datalist is None:
            pcmd = None
        else:
            pcmd = pickle.dumps(datalist)

        hdr = bytes(bytearray([cmd]))

        if pcmd:
            hdr += struct.pack("!I", len(pcmd))
            cmdpacket = hdr + pcmd
        else:
            hdr += struct.pack("!I", 0)
            cmdpacket = hdr

        return cmdpacket

class NAEUSB_Serializer(NAEUSB_Serializer_base):
    """
    This class does not talk to libusb directly, but instead uses a class that expects special commands passed
    to it. The function which will process commands must be passed to the initilizer, which allows passing the
    data over a network or similar.
    """

    def __init__(self, transmitfunc):

        self.txrx = transmitfunc


    def get_possible_devices(self, idProduct=None):
        """Get a list of connected USB devices."""
        cmdpacket = self.make_cmd(self.GET_POSSIBLE_DEVICES, idProduct)
        return self.process_rx(self.txrx(tx=cmdpacket))

    def process_rx(self, inp):
        """Process the data received back from the end, normally means find errors or return payload."""
        resp = inp[0]

        plen = struct.unpack("!I", inp[1:5])[0]

        if plen > 0:

            pdata = inp[5:]
            if plen != len(pdata):
                raise ValueError("This pickle smells funny")

            payload = pickle.loads(pdata)
        else:
            payload = None


        if resp == self.ERROR:
            raise payload

        return payload

    def open(self, serial_number = None):
        """Opens USB device"""
        cmdpacket = self.make_cmd(self.OPEN, serial_number)
        return self.process_rx(self.txrx(tx=cmdpacket))

    def close(self, serial_number):
        """Close USB connection."""
        cmdpacket = self.make_cmd(self.CLOSE, serial_number)
        return self.process_rx(self.txrx(tx=cmdpacket))

    def sendCtrl(self, cmd, value=0, data=[]):
        """
        Send data over control endpoint
        """
        # Vendor-specific, OUT, interface control transfer

        cmdpacket = [0x41, cmd, value, 0, len(data)]
        cmdpacket.extend(data)

        cmdpacket = self.make_cmd(self.WRITE_CTRL, cmdpacket)

        self.process_rx(self.txrx(tx=cmdpacket))

    def readCtrl(self, cmd, value=0, dlen=0):
        """
        Read data from control endpoint
        """

        cmdpacket = [0xC1, cmd, value, 0, dlen]
        cmdpacket = self.make_cmd(self.READ_CTRL, cmdpacket)
        return self.process_rx(self.txrx(tx=cmdpacket))


    def cmdReadMem(self, addr, dlen):
        """
        Send command to read over external memory interface from FPGA. Automatically
        decides to use control-transfer or bulk-endpoint transfer based on data length.
        """

        dlen = int(dlen)
        payload = [addr, dlen]
        cmdpacket = self.make_cmd(self.CMD_READ_MEM, payload)

        return self.process_rx(self.txrx(tx=cmdpacket))

    def cmdWriteMem(self, addr, data):
        """
        Send command to write memory over external memory interface to FPGA. Automatically
        decides to use control-transfer or bulk-endpoint transfer based on data length.
        """

        dlen = len(data)

        payload = [addr]
        payload.extend(data)
        cmdpacket = self.make_cmd(self.CMD_WRITE_MEM, payload)
        return self.process_rx(self.txrx(tx=cmdpacket))

    def writeBulk(self, data):
        """
        Low-level function.
        Writes bulk data to the bulk USB endpoint.
        :param data: Data to write to the endpoint
        :return:
        """
        cmdpacket = self.make_cmd(self.WRITE_BULK, data)
        return self.process_rx(self.txrx(tx=cmdpacket))

    def flushInput(self):
        """
        Of dubious value: flushes the USB endpoints. Causes slowdowns on MAC OS X, so need to investigate
        the usefulness of this.
        """

        cmdpacket = self.make_cmd(self.FLUSH_INPUT, None)
        return self.process_rx(self.txrx(tx=cmdpacket))


class NAEUSB_Backend(NAEUSB_Serializer_base):
    """
    This backend actually talks to the USB device itself. It is designed to mostly be used via the serializer, but
    can be called directly too.
    """

    CMD_READMEM_BULK = 0x10
    CMD_WRITEMEM_BULK = 0x11
    CMD_READMEM_CTRL = 0x12
    CMD_WRITEMEM_CTRL = 0x13
    CMD_MEMSTREAM = 0x14

    def __init__(self):
        self._usbdev = None
        self._timeout = 500

    def usbdev(self):
        """Safely get USB device, throwing error if not connected"""

        if not self._usbdev: raise OSError("USB Device not found. Did you connect it first?")
        return self._usbdev

    def txrx(self, tx=[]):
        """
        Process USB command, and returns a result such as data or an encoded exception. Excepts that happen
        are not raised, but instead only printed (for debug) and passed back.
        """

        response = None

        try:
            #Get command
            cmd = tx[0]
            pickle_len = struct.unpack("!I", tx[1:5])[0]

            if pickle_len > 0:
                pickle_data = tx[5:]

                if len(pickle_data) != pickle_len:
                    raise ValueError("Pickle smells funny. Check best before date.")

                payload = pickle.loads(pickle_data)
            else:
                payload = None

            if cmd == self.READ_CTRL:
                response = self.usbdev().ctrl_transfer(payload[0], payload[1], payload[2], payload[3], payload[4], timeout=self._timeout)
            elif cmd == self.WRITE_CTRL:
                if payload[4] != len(payload[5:]):
                    raise ValueError("Specified payload length & actual do not match")
                self.usbdev().ctrl_transfer(payload[0], payload[1], payload[2], payload[3], payload[5:], timeout=self._timeout)
            elif cmd == self.CMD_READ_MEM:
                addr = payload[0]
                dlen = payload[1]
                response = self.cmdReadMem(addr, dlen)
            elif cmd == self.CMD_WRITE_MEM:
                addr = payload[0]
                data = payload[1:]
                self.cmdWriteMem(addr, data)
            elif cmd == self.GET_POSSIBLE_DEVICES:
                response = self.get_possible_devices(payload)
            elif cmd == self.OPEN:
                response = self.open(serial_number=payload)
            elif cmd == self.CLOSE:
                self.close()
            elif cmd == self.WRITE_BULK:
                self.cmdWriteBulk(payload)
            elif cmd == self.FLUSH_INPUT:
                self.flushInput()
            elif cmd == self.READ:
                dlen = payload[0]
                response = self.read(dlen)
            else:
                raise ValueError("Unknown Command: %02x"%cmd)
        except Exception as e:
            traceback.print_exc()
            return self.make_cmd(self.ERROR, e)

        return self.make_cmd(self.ACK, response)

    def open(self, serial_number=None, connect_to_first=False):
        """
        Connect to device using default VID/PID
        """

        devlist = self.get_possible_devices(dictonly=False)
        snlist = [d.serial_number + " (" + d.product + ")\n" for d in devlist]
        snlist = "".join(snlist)

        if len(devlist) == 0:
            raise OSError("Failed to find USB Device")

        elif serial_number:
            dev = None
            for d in devlist:
                if d.serial_number == serial_number:
                    dev = d
                    break

            if dev is None:
                raise OSError("Failed to find USB device with S/N %s\n. Found S/N's:\n" + snlist)

        elif len(devlist) == 1:
            dev = devlist[0]

        else:
            if connect_to_first:
                dev = devlist[0]
            else:
                #User did not help us out - throw it in their face
                raise Warning("Found multiple potential USB devices. Please specify device to use. Possible S/Ns:\n" + snlist)
        try:
            dev.set_configuration(0)
            dev.set_configuration()
        except ValueError:
            raise OSError("NAEUSB: Could not configure USB device")

        # Get serial number
        try:
            # New calling syntax
            self.snum = usb.util.get_string(dev, index=3)

        except TypeError:
            # Old calling syntax
            self.snum = usb.util.get_string(dev, length=256, index=3)

        foundId = dev.idProduct

        if foundId in NEWAE_PIDS:
            name = NEWAE_PIDS[foundId]['name']
            fw_latest = NEWAE_PIDS[foundId]['fwver']
        else:
            name = "Unknown (PID = %04x)"%foundId
            fw_latest = [0, 0]

        logging.info('Found %s, Serial Number = %s' % (name, self.snum))

        self._usbdev = dev
        self.rep = 0x81
        self.wep = 0x02
        self._timeout = 200

        return foundId

    def close(self):
        """Close the USB connection"""
        try:
            usb.util.dispose_resources(self.usbdev())
        except usb.USBError as e:
            logging.info('USB Failure calling dispose_resources: %s' % str(e))


    def get_possible_devices(self, idProduct=None, dictonly=True):
        """
        Get a list of matching devices being based a list of PIDs. Returns list of usbdev that match (or empty if none)
        """
        if idProduct is None:
            idProduct = [None]

        libusb_backend = libusb0.get_backend()
        if os.name == "nt":
            #on windows, need to manually load libusb because of 64bit python loading the wrong one
            libusb_backend = libusb0.get_backend(find_library=lambda x: r"c:\Windows\System32\libusb0.dll")
        devlist = []
        try:
            for id in idProduct:
                if id:
                    dev = list(usb.core.find(find_all=True, idVendor=0x2B3E, idProduct=id, backend=libusb_backend))
                else:
                    dev = list(usb.core.find(find_all=True, idVendor=0x2B3E, backend=libusb_backend))
                if len(dev) > 0:
                    if len(dev) == 1:
                        devlist.extend(dev)
                    else:
                        # Deals with the multiple chipwhisperers attached but user only
                        # has permission to access a subset. The langid error is usually
                        # raised when there are improper permissions, so it is used to
                        # skip the those devices. However, the user is warned when this
                        # happens because the langid error is occasionally raised when
                        # there are backend errors.
                        for d in dev:
                            try:
                                d.serial_number
                                devlist.append(d)
                            except ValueError as e:
                                if "langid" in str(e):
                                    logging.info('A device raised the "no langid" error, it is being skipped')
                                else:
                                    raise
            if dictonly:
                devlist = [{'sn': d.serial_number, 'product': d.product, 'pid': d.idProduct, 'vid': d.idVendor} for d in devlist]

            return devlist
        except ValueError as e:
            if "langid" not in str(e):
                raise
            raise OSError("'This device has no langid' ValueError caught. This is usually caused by us trying to read the serial number of the chipwhisperer, but it failing. The device is here and we can see it, but we can't access it. This has a number of root causes, including:\n" +
                        "-Not having permission to access the ChipWhisperer (this still crops up if you have permission for one ChipWhisperer, but another ChipWhisperer is connected that you don't have access to)\n" +
                        "-Not having the correct libusb backend loaded (common on Windows with 64bit Python). We try to handle this by loading the correct backend on Windows"
                        )

    def sendCtrl(self, cmd, value=0, data=[]):
        """
        Send data over control endpoint
        """
        # Vendor-specific, OUT, interface control transfer
        return self.usbdev().ctrl_transfer(0x41, cmd, value, 0, data, timeout=self._timeout)

    def readCtrl(self, cmd, value=0, dlen=0):
        """
        Read data from control endpoint
        """
        # Vendor-specific, IN, interface control transfer
        return self.usbdev().ctrl_transfer(0xC1, cmd, value, 0, dlen, timeout=self._timeout)

    def cmdReadMem(self, addr, dlen):
        """
        Send command to read over external memory interface from FPGA. Automatically
        decides to use control-transfer or bulk-endpoint transfer based on data length.
        """

        dlen = int(dlen)

        if dlen < 48:
            cmd = self.CMD_READMEM_CTRL
        else:
            cmd = self.CMD_READMEM_BULK

        # ADDR/LEN written LSB first
        pload = packuint32(dlen)
        pload.extend(packuint32(addr))
        self.sendCtrl(cmd, data=pload)

        # Get data
        if cmd == self.CMD_READMEM_BULK:
            data = self.usbdev().read(self.rep, dlen, timeout=self._timeout)
        else:
            data = self.readCtrl(cmd, dlen=dlen)

        return data

    def cmdWriteMem(self, addr, data):
        """
        Send command to write memory over external memory interface to FPGA. Automatically
        decides to use control-transfer or bulk-endpoint transfer based on data length.
        """

        dlen = len(data)

        if dlen < 48:
            cmd = self.CMD_WRITEMEM_CTRL
        else:
            cmd = self.CMD_WRITEMEM_BULK

        # ADDR/LEN written LSB first
        pload = packuint32(dlen)
        pload.extend(packuint32(addr))

        if cmd == self.CMD_WRITEMEM_CTRL:
            pload.extend(data)

        self.sendCtrl(cmd, data=pload)

        # Get data
        if cmd == self.CMD_WRITEMEM_BULK:
            data = self.usbdev().write(self.wep, data, timeout=self._timeout)
        else:
            pass

        return data


    def cmdWriteBulk(self, data):
        """
        Write data directly to the bulk endpoint.
        :param data: Data to be written
        :return:
        """

        self.usbdev().write(self.wep, data, timeout=self._timeout)


    def flushInput(self):
        """Dump all the crap left over"""
        try:
            # TODO: This probably isn't needed, and causes slow-downs on Mac OS X.
            self.usbdev().read(self.rep, 1000, timeout=0.010)
        except:
            pass

    def read(self, dbuf, timeout):
        return self.usbdev().read(self.rep, dbuf, timeout)

class NAEUSB(object):
    """
    USB Interface for NewAE Products with Custom USB Firmware. This function allows use of a daemon backend, as it is
    not directly touching the USB device itself.
    """

    CMD_FW_VERSION = 0x17

    CMD_READMEM_BULK = 0x10
    CMD_WRITEMEM_BULK = 0x11
    CMD_READMEM_CTRL = 0x12
    CMD_WRITEMEM_CTRL = 0x13
    CMD_MEMSTREAM = 0x14

    stream = False

    # TODO: make this better
    fwversion_latest = [0, 11]
    def __init__(self):
        self._usbdev = None
        self.usbtx = NAEUSB_Backend()
        self.usbseralizer = NAEUSB_Serializer(self.usbtx.txrx)

    def get_possible_devices(self, idProduct):
        return self.usbseralizer.get_possible_devices(idProduct)

    def con(self, idProduct=[0xACE2], connect_to_first=False, serial_number=None):
        """
        Connect to device using default VID/PID
        """

        devlist = self.get_possible_devices(idProduct)

        snlist = [d['sn'] + " (" + d['product'] + ")\n" for d in devlist]
        snlist = "".join(snlist)

        if len(devlist) == 0:
            raise Warning("Failed to find USB Device")

        elif serial_number:
            dev = None
            for d in devlist:
                if d["sn"] == serial_number:
                    dev = d
                    break

            if dev is None:
                raise Warning("Failed to find USB device with S/N %s\n. Found S/N's:\n" + snlist)

        elif len(devlist) == 1:
            dev = devlist[0]

        else:
            if connect_to_first:
                dev = devlist[0]
            else:
                #User did not help us out - throw it in their face
                raise Warning("Found multiple potential USB devices. Please specify device to use. Possible S/Ns:\n" + snlist)

        self.usbseralizer.open(dev['sn'])
        foundId = dev['pid']

        if foundId in NEWAE_PIDS:
            name = NEWAE_PIDS[foundId]['name']
            fw_latest = NEWAE_PIDS[foundId]['fwver']
        else:
            name = "Unknown (PID = %04x)"%foundId
            fw_latest = [0, 0]

        self.snum = dev['sn']

        logging.info('Found %s, Serial Number = %s' % (name, self.snum))

        fwver = self.readFwVersion()
        logging.info('SAM3U Firmware version = %d.%d b%d' % (fwver[0], fwver[1], fwver[2]))

        latest = fwver[0] > fw_latest[0] or (fwver[0] == fw_latest[0] and fwver[1] >= fw_latest[1])
        if not latest:
            logging.warning('Your firmware is outdated - latest is %d.%d' % (fw_latest[0], fw_latest[1]) +
                            '. Suggested to update firmware, as you may experience errors')
        return foundId

    def usbdev(self):
        raise AttributeError("Do Not Call Me")

    def close(self):
        """Close USB connection."""
        self.usbseralizer.close(self.snum)
        self.snum = None

    def readFwVersion(self):
        try:
            data = self.readCtrl(self.CMD_FW_VERSION, dlen=3)
            return data
        except usb.USBError:
            return [0, 0, 0]

    def sendCtrl(self, cmd, value=0, data=[]):
        """
        Send data over control endpoint
        """
        # Vendor-specific, OUT, interface control transfer
        self.usbseralizer.sendCtrl(cmd, value, data)

    def readCtrl(self, cmd, value=0, dlen=0):
        """
        Read data from control endpoint
        """
        # Vendor-specific, IN, interface control transfer
        return self.usbseralizer.readCtrl(cmd, value, dlen)

    def cmdReadMem(self, addr, dlen):
        """
        Send command to read over external memory interface from FPGA. Automatically
        decides to use control-transfer or bulk-endpoint transfer based on data length.
        """

        return self.usbseralizer.cmdReadMem(addr, dlen)

    def cmdWriteMem(self, addr, data):
        """
        Send command to write memory over external memory interface to FPGA. Automatically
        decides to use control-transfer or bulk-endpoint transfer based on data length.
        """

        return self.usbseralizer.cmdWriteMem(addr, data)

    def writeBulkEP(self, data):
        """
        Write directoly to the bulk endpoint.
        :param data: Data to be written.
        :return:
        """

        return self.usbseralizer.writeBulk(data)

    def flushInput(self):
        """Dump all the crap left over"""
        self.usbseralizer.flushInput()

    def cmdReadStream_getStatus(self):
        """
        Gets the status of the streaming mode capture, tells you samples left to stream out along
        with overflow buffer status. When an overflow occurs the samples left to stream goes to
        zero.

        samples_left_to_stream is number of samples not yet streamed out of buffer.
        overflow_lcoation is the value of samples_left_to_stream when a buffer overflow occured.
        unknown_overflow is a flag indicating if an overflow occured at an unknown time.

        Returns:
            Tuple indicating (samples_left_to_stream, overflow_location, unknown_overflow)
        """
        data = self.readCtrl(self.CMD_MEMSTREAM, dlen=9)

        status = data[0]
        samples_left_to_stream = unpackuint32(data[1:5])
        overflow_location = unpackuint32(data[5:9])

        if status == 0:
            unknown_overflow = False
        else:
            unknown_overflow = True

        return (samples_left_to_stream, overflow_location, unknown_overflow)

    def cmdReadStream_size_of_fpgablock(self):
        """ Asks the hardware how many BYTES are read in one go from FPGA, which indicates where the sync
            bytes will be located. These sync bytes must be removed in post-processing. """
        return 4096

    def cmdReadStream_bufferSize(self, dlen):
        """
        Args:
            dlen: Number of samples to be requested (will be rounded to something else)

        Returns:
            Tuple: (Size of temporary buffer required, actual samples in buffer)
        """
        num_samplebytes = int(math.ceil(float(dlen) * 4 / 3))
        num_blocks = int(math.ceil(float(num_samplebytes) / 4096))
        num_totalbytes = num_samplebytes + num_blocks
        num_totalbytes = int(math.ceil(float(num_totalbytes) / 4096) * 4096)
        return (num_totalbytes, num_samplebytes)


    def initStreamModeCapture(self, dlen, dbuf_temp, timeout_ms=1000):
        #Enter streaming mode for requested number of samples
        if hasattr(self, "streamModeCaptureStream"):
            self.streamModeCaptureStream.join()
        self.sendCtrl(NAEUSB.CMD_MEMSTREAM, data=packuint32(dlen))
        self.streamModeCaptureStream = NAEUSB.StreamModeCaptureThread(self, dlen, dbuf_temp, timeout_ms)
        self.streamModeCaptureStream.start()

    def cmdReadStream_isDone(self):
        return self.streamModeCaptureStream.isAlive() == False

    def cmdReadStream(self):
        """
        Gets data acquired in streaming mode.
        initStreamModeCapture should be called first in order to make it work.
        """
        self.streamModeCaptureStream.join()

        # Flush input buffers in case anything was left
        try:
            #self.cmdReadMem(self.rep)
            self.usbtx.read(4096, timeout=10)
            self.usbtx.read(4096, timeout=10)
            self.usbtx.read(4096, timeout=10)
            self.usbtx.read(4096, timeout=10)
        except IOError:
            pass

        # Ensure stream mode disabled
        self.sendCtrl(NAEUSB.CMD_MEMSTREAM, data=packuint32(0))

        return self.streamModeCaptureStream.drx, self.streamModeCaptureStream.timeout

    def enterBootloader(self, forreal=False):
        """Erase the SAM3U contents, forcing bootloader mode. Does not screw around."""

        if forreal:
            self.sendCtrl(0x22, 3)

    def read(self, dlen, timeout=2000):
        self.usbserializer.read(dlen, timeout)

    class StreamModeCaptureThread(Thread):
        def __init__(self, serial, dlen, dbuf_temp, timeout_ms=2000):
            """
            Reads from the FIFO in streaming mode. Requires the FPGA to be previously configured into
            streaming mode and then arm'd, otherwise this may return incorrect information.

            Args:
                dlen: Number of samples to request.
                dbuf_temp: Temporary data buffer, must be of size cmdReadStream_bufferSize(dlen) or bad things happen
                timeout_ms: Timeout in ms to wait for stream to start, otherwise returns a zero-length buffer
            Returns:
                Tuple of (samples_per_block, total_bytes_rx)
            """
            Thread.__init__(self)
            self.dlen = dlen
            self.dbuf_temp = dbuf_temp
            self.timeout_ms = timeout_ms
            self.serial = serial
            self.timeout = False
            self.drx = 0

        def run(self):
            logging.debug("Streaming: starting USB read")
            start = time.time()
            try:
                self.drx = self.serial.usbtx.read(self.dbuf_temp, timeout=self.timeout_ms)
            except IOError as e:
                logging.warning('Streaming: USB stream read timed out')
            diff = time.time() - start
            logging.debug("Streaming: Received %d bytes in time %.20f)" % (self.drx, diff))


if __name__ == '__main__':
    from chipwhisperer.hardware.naeusb.fpga import FPGA
    from chipwhisperer.hardware.naeusb.programmer_avr import AVRISP
    from chipwhisperer.hardware.naeusb.programmer_xmega import XMEGAPDI, supported_xmega
    from chipwhisperer.hardware.naeusb.serial import USART

    cwtestusb = NAEUSB()
    cwtestusb.con()

    #Connect required modules up here
    fpga = FPGA(cwtestusb)
    xmega = XMEGAPDI(cwtestusb)
    avr = AVRISP(cwtestusb)
    usart = USART(cwtestusb)

    force = True
    if fpga.isFPGAProgrammed() == False or force:
        from datetime import datetime
        starttime = datetime.now()
        fpga.FPGAProgram(open(r"C:\E\Documents\academic\sidechannel\chipwhisperer\hardware\capture\chipwhisperer-lite\hdl\cwlite_ise\cwlite_interface.bit", "rb"))
        # fpga.FPGAProgram(open(r"C:\Users\colin\dropbox\engineering\git_repos\CW305_ArtixTarget\temp\artix7test\artix7test.runs\impl_1\cw305_top.bit", "rb"))
        # fpga.FPGAProgram(open(r"C:\E\Documents\academic\sidechannel\chipwhisperer\hardware\api\chipwhisperer-lite\hdl\cwlite_ise_spifake\cwlite_interface.bit", "rb"))
        stoptime = datetime.now()
        print("FPGA Config time: %s" % str(stoptime - starttime))

    # print fpga.cmdReadMem(10, 6)
    # print fpga.cmdReadMem(0x1A, 4)
    # fpga.cmdWriteMem(0x1A, [235, 126, 5, 4])
    # print fpga.cmdReadMem(0x1A, 4)

    avrprogram = False
    if avrprogram:
        avr.enableISP(True)
        avr.enableISP(False)

    xmegaprogram = True
    if xmegaprogram:
        xmega.setChip(supported_xmega[0])
        # Worst-case is 75mS for chip erase, so give us some head-room
        xmega.setParamTimeout(200)

        try:
            print("Enable")
            xmega.enablePDI(True)

            print("Read sig")
            # Read signature bytes
            data = xmega.readMemory(0x01000090, 3, "signature")

            print(data)

            if data[0] != 0x1E or data[1] != 0x97 or data[2] != 0x46:
                print("Signature bytes failed: %02x %02x %02x != 1E 97 46" % (data[0], data[1], data[2]))
            else:
                print("Detected XMEGA128A4U")

            print("Erasing")
            # Chip erase
            try:
                xmega.eraseChip()
            except IOError:
                xmega.enablePDI(False)
                xmega.enablePDI(True)

            fakedata = [i & 0xff for i in range(0, 2048)]
            print("Programming FLASH Memory")
            xmega.writeMemory(0x0800000, fakedata, memname="flash")

            print("Verifying")
            test = xmega.readMemory(0x0800000, 512)

            print(test)


        except TypeError as e:
            print(str(e))

        except IOError as e:
            print(str(e))

        xmega.enablePDI(False)

    print("Let's Rock and Roll baby")

    sertest = True

    if sertest:
        usart.init()
        usart.write("hello\n")
        time.sleep(0.1)
        print(usart.read())



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import numpy as np
import os
import sys
from ctypes import *

from ..algorithmsbase import AlgorithmsBase


class aesmodel_setup_t(Structure):

    TARGET_SBOXOUT_HW = 1
    TARGET_TARGET_INVSBOX_LASTROUND_HD = 2

    _fields_ = [("bnum", c_uint),
                ("roundkeys", POINTER(c_uint8)),
                ("rtarget", c_uint),
                ("leakagemode", c_uint),
                ("privatedata", c_void_p),
                ("privatedatasize", c_uint)
                ]
            
    def __init__(self, bnum=0):  
        super(aesmodel_setup_t, self).__init__()
        self.bnum = c_uint(bnum)
        self.rtarget = c_uint(0)
        self.privatedatasize = c_uint(0)
        self.leakagemode = c_uint(self.TARGET_SBOXOUT_HW)


class analysis_state_t(Structure):
    _fields_=[("sumhq",POINTER(c_double)),
              ("sumtq",POINTER(c_double)),
            ("sumt",POINTER(c_double)),
            ("sumh",POINTER(c_double)),
            ("sumht",POINTER(c_double)),
            ("totalTraces",c_int),
            ("hyp",POINTER(c_double))]
            
    def __init__(self, npoint=0, ntrace=0):  
        super(analysis_state_t,self).__init__()
      
        nguess = 256
      
        self._sumhq = np.zeros(nguess, dtype=np.float64)
        self.sumhq = self._sumhq.ctypes.data_as(POINTER(c_double))
      
        self._sumtq = np.zeros(npoint, dtype=np.float64)
        self.sumtq = self._sumtq.ctypes.data_as(POINTER(c_double))
      
        self._sumt= np.zeros(npoint, dtype=np.float64)
        self.sumt = self._sumt.ctypes.data_as(POINTER(c_double))
      
        self._sumh = np.zeros(nguess, dtype=np.float64)
        self.sumh = self._sumh.ctypes.data_as(POINTER(c_double))
      
      
        self._sumht = np.zeros((nguess, npoint), dtype=np.float64)
        self.sumht = self._sumht.ctypes.data_as(POINTER(c_double))
      
        self.totalTraces = c_int(0)

        self._hyp = np.zeros(ntrace, dtype=np.float64)
        self.hyp = self._hyp.ctypes.data_as(POINTER(c_double))


c_analysis_state_t_ptr = POINTER(analysis_state_t)
c_aesmodel_setup_t_ptr = POINTER(aesmodel_setup_t)


class CPAProgressiveOneSubkey(object):
    """This class is the basic progressive CPA attack, capable of adding traces onto a variable with previous data"""
    def __init__(self):
        self.clearStats()
        dir = os.path.dirname(__file__)
        
        #Determine correct library to load for 64-bit vs. 32-bit

        is_64bits = sys.maxsize > 2 ** 32

        if os.name == 'nt':
            if is_64bits:
                libname = 'libcpa_x64.dll'
            else:
                libname = 'libcpa.dll'
        elif os.name == 'posix':
            if is_64bits:
                libname = 'libcpa_x64.so'
            else:
                libname = 'libcpa.so'

        libname = os.path.join(dir, 'c_accel/%s' % libname)
        try:
            dll = CDLL(libname)
        except Exception:
            raise Exception("Could not import library file. Compile it for your platform first (there is a makefile for that): " + libname)

        self.osk = dll.oneSubkey
        self.modelstate = {'knownkey':None}

    def clearStats(self):
        self.anstate = None

    def oneSubkey(self, bnum, pointRange, traces_all, numtraces, plaintexts, ciphertexts, knownkeys, progressBar, model, state, pbcnt):

        if pointRange == None:
            traces = traces_all
            pointstart = 0
        else:
            # traces = np.array(traces_all[:, pointRange[0] : pointRange[1]])
            traces = traces_all[:, pointRange[0] : pointRange[1]]
            pointstart = pointRange[0]


        if (pointRange[0] != 0):
            raise NotImplementedError("C-Accel only works with full point range on current version.")

        #TODO: Captured traces seem to have extra point, this is temp fix as need to figure out why this is
        #      happening!?
        npoints = np.shape(traces)[1]+1

        if self.anstate is None:
            self.anstate = analysis_state_t(npoints, numtraces)
            
        mstate = aesmodel_setup_t(bnum=bnum)
        
        guessdata = np.zeros((model.getPermPerSubkey(), npoints-pointstart), dtype=np.float64)

        if hasattr(model.getHwModel(), 'c_model_enum_value'):
            mstate.leakagemode = model.getHwModel().c_model_enum_value
        else:
            mstate.leakagemode = model.getHwModel()

        self.osk(traces.ctypes.data_as(POINTER(c_double)),
                 plaintexts.ctypes.data_as(POINTER(c_uint8)),
                 ciphertexts.ctypes.data_as(POINTER(c_uint8)),
                 c_size_t(len(traces)),
                 c_size_t(npoints),
                 c_size_t(0),
                 c_size_t(numtraces),
                 c_size_t(pointstart),
                 c_size_t(npoints),
                 c_analysis_state_t_ptr(self.anstate),
                 c_void_p(0),
                c_aesmodel_setup_t_ptr(mstate),
                 guessdata.ctypes.data_as(POINTER(c_double)))
      
        if progressBar:
            progressBar.updateStatus(pbcnt, (self.anstate.totalTraces - numtraces, self.anstate.totalTraces-1, bnum))

        pbcnt = pbcnt + model.getPermPerSubkey()

        return (guessdata, pbcnt)


class CPAProgressive_CAccel(AlgorithmsBase):
    """
    CPA Attack done as a loop, but using an algorithm which can progressively add traces & give output stats
    """
    _name = "Progressive-C Accel"

    def __init__(self):
        AlgorithmsBase.__init__(self)

        self.getParams().addChildren([
            {'name':'Iteration Mode', 'key':'itmode', 'type':'list', 'values':{'Depth-First':'df', 'Breadth-First':'bf'}, 'value':'bf'},
            {'name':'Skip when PGE=0', 'key':'checkpge', 'type':'bool', 'value':False},
        ])
        self.updateScript()


    def addTraces(self, traceSource, tracerange, progressBar=None, pointRange=None):
        brange=self.brange

        numtraces = tracerange[1] - tracerange[0]

        if progressBar:
            progressBar.setText("Attacking traces: from %d to %d (total = %d)" % (tracerange[0], tracerange[1], numtraces))
            progressBar.setStatusMask("Trace Interval: %d-%d. Current Subkey: %d", (0,0,0))
            progressBar.setMaximum(len(brange) * self.model.getPermPerSubkey() * (numtraces / self._reportingInterval + 1))
        pbcnt = 0
        #r = Parallel(n_jobs=4)(delayed(traceOneSubkey)(bnum, pointRange, traces_all, numtraces, plaintexts, ciphertexts, keyround, modeltype, progressBar, self.model, pbcnt) for bnum in brange)
        #self.all_diffs, pb = zip(*r)
        #pbcnt = 0
        cpa = [None]*(max(brange)+1)
        for bnum in brange:
            cpa[bnum] = CPAProgressiveOneSubkey()

        brangeMap = [None]*(max(brange)+1)
        i = 1
        for bnum in brange:
            brangeMap[bnum] = i
            i += 1

        skipPGE = False  # self.findParam('checkpge').getValue()
        bf = True  # self.findParam('itmode').getValue() == 'bf'

        #bf specifies a 'breadth-first' search. bf means we search across each
        #subkey by only the amount of traces specified. Depth-First means we
        #search each subkey completely, then move onto the next.
        if bf:
            brange_df = [0]
            brange_bf = brange
        else:
            brange_bf = [0]
            brange_df = brange


        for bnum_df in brange_df:

            tstart = 0
            tend = self._reportingInterval

            while tstart < numtraces:
                if tend > numtraces:
                    tend = numtraces

                if tstart > numtraces:
                    tstart = numtraces

                data = []
                textins = []
                textouts = []
                knownkeys = []
                for i in range(tstart, tend):
                    # Handle Offset
                    tnum = i + tracerange[0]

                    d = traceSource.getTrace(tnum)

                    if d is None:
                        continue

                    data.append(d)
                    textins.append(traceSource.getTextin(tnum))
                    textouts.append(traceSource.getTextout(tnum))
                    knownkeys.append(traceSource.getKnownKey(tnum))

                traces = np.array(data)
                textins = np.array(textins)
                textouts = np.array(textouts)

                for bnum_bf in brange_bf:

                    if bf:
                        bnum = bnum_bf
                    else:
                        bnum = bnum_df


                    skip = False
                    if (self.stats.simplePGE(bnum) != 0) or (skipPGE == False):
                        if isinstance(pointRange, list):
                            bptrange = pointRange[bnum]
                        else:
                            bptrange = pointRange
                        (data, pbcnt) = cpa[bnum].oneSubkey(bnum, bptrange, traces, tend - tstart, textins, textouts, knownkeys, progressBar, self.model, cpa[bnum].modelstate, pbcnt)
                        self.stats.updateSubkey(bnum, data, tnum=tend)
                    else:
                        skip = True

                    if skip:
                        pbcnt = brangeMap[bnum] * self.model.getPermPerSubkey() * (numtraces / self._reportingInterval + 1)

                        if bf is False:
                            tstart = numtraces

                tend += self._reportingInterval
                tstart += self._reportingInterval

                if self.sr is not None:
                    self.sr()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging
import random
from chipwhisperer.common.utils import util
from ._base import AcqKeyTextPattern_Base

try:
    from Crypto.Cipher import AES
except ImportError:
    logging.warn('T-test inputs: no AES module found. Using pseudorandom plaintexts instead')
    AES = None


class AcqKeyTextPattern_TVLATTest(AcqKeyTextPattern_Base):
    """Class for getting key and text for TVLA T-Tests.

    Basic usage::

        import chipwhisperer as cw
        ktp = cw.ktp.TVLATTest()
        ktp.init(num_traces) # init with the number of traces you plan to
                             # capture
        key, text = ktp.next()

    """
    _name = "TVLA Rand vs Fixed"
    _description = "Welsh T-Test with random/fixed plaintext."

    def __init__(self, target=None):
        AcqKeyTextPattern_Base.__init__(self)
        self._interleavedPlaintext = []
        self._key = []


        self.setTarget(target)

    def _initPattern(self):
        pass

    def init(self, maxtraces):
        """Initialize key text pattern for a specific number of traces.

        Args:
            maxtraces (int): Number of traces to initialize for.

        Raises:
            ValueError: Invalid key length
        """
        length = self.keyLen()
        if length <= 32:
            self._key = util.hexStrToByteArray("01 23 45 67 89 ab cd ef 12 34 56 78 9a bc de f0 23 45 67 89 ab cd ef 01 34 56 78 9a bc de f0 12")[:length]
        else:
            raise ValueError("Invalid key length: %d bytes" % length)

        self._textin1 = util.hexStrToByteArray("00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00")

        if length == 16:
            self._interleavedPlaintext = util.hexStrToByteArray("da 39 a3 ee 5e 6b 4b 0d 32 55 bf ef 95 60 18 90")
        elif length == 24:
            self._interleavedPlaintext = util.hexStrToByteArray("da 39 a3 ee 5e 6b 4b 0d 32 55 bf ef 95 60 18 88")
        elif length == 32:
            self._interleavedPlaintext = util.hexStrToByteArray("da 39 a3 ee 5e 6b 4b 0d 32 55 bf ef 95 60 18 95")
        else:
            raise ValueError("Invalid key length: %d bytes" % length)

        self.num_group1 = int(maxtraces/2)
        self.num_group2 = int(maxtraces - self.num_group1)

    def new_pair(self):
        rand = random.random()
        num_tot = self.num_group1 + self.num_group2
        if num_tot == 0:
            group1 = (rand < 0.5)
        else:
            cutoff = float(self.num_group1) / num_tot
            group1 = (rand < cutoff)

        if group1:
            self._textin = self._textin1

            if AES is not None:
                cipher = AES.new(bytes(self._key), AES.MODE_ECB)
                self._textin1 = bytearray(cipher.encrypt(bytes(self._textin1)))
            else:
                self._textin1 = bytearray(16)
                for i in range(0, 16):
                    self._textin1[i] = random.randint(0, 255)
                if self.num_group1 > 0:
                    self.num_group1 -= 1
        else:
            self._textin = self._interleavedPlaintext
            if self.num_group2 > 0:
                self.num_group2 -= 1

        # Check key works with target
        self.validateKey()

        return self._key, self._textin

    def next(self):
        """Returns the next key text pair

        Updates last key and text

        Returns:
            (key (bytearray), text (bytearray))

        .. versionadded:: 5.1
            Added next
        """
        return self.new_pair()
